import asyncio
import time
from dataclasses import dataclass, field
from pathlib import Path
from typing import Optional, List, Dict, Any
from ErisPulse import sdk


@dataclass
class TestConfig:
    """测试配置类"""
    # 适配器配置
    adapter_name: str = "yunhu_user"  # 要测试的适配器名称
    
    # 基础配置
    group_id: str = "272475188"  # 测试群号
    test_user_id: str = "5197892"  # 测试用户ID
    test_user_id_2: str = "5940358"  # 第二个测试用户ID
    
    # 文件路径配置
    test_files_dir: str = "test_files"
    video_file: str = "test.mp4"
    image_file: str = "test.jpg"
    doc_file  : str = "test.docx"
    voice_file: Optional[str] = "M5000017K7gL4WYnw2.mp3"
    
    # 消息发送间隔（秒）
    send_interval: float = 1.0
    
    # 启用/禁用特定测试
    enable_basic_tests: bool = True  # 基础测试（1-5）
    enable_media_tests: bool = True  # 媒体测试（6-11）
    enable_advanced_tests: bool = True  # 高级功能测试（12-16）
    enable_format_tests: bool = True  # 格式化消息测试（17-19）
    enable_chain_tests: bool = True  # 链式调用测试（20-24）
    enable_mention_tests: bool = True  # @功能测试（25-26）
    
    # 单独启用的测试（列表形式）
    # 例如：[1, 6, 7] 表示只运行测试1、6和7
    # 如果为空列表，则根据上面的 bool 配置运行
    specific_tests: List[int] = field(default_factory=list)
    # specific_tests = [12, 13, 14, 21, 22]
    
    # URL 配置
    image_url: str = "https://http.cat/200"
    voice_url: str = "https://download.samplelib.com/mp3/sample-12s.mp3"
    video_url: str = "https://www.w3school.com.cn/example/html5/mov_bbb.mp4"
    file_url: str = "https://www.w3school.com.cn/example/html5/mov_bbb.mp4"


@dataclass
class TestResult:
    """单个测试结果"""
    test_num: int
    test_name: str
    status: str  # "success", "failed", "error", "skipped"
    response: Optional[dict] = None
    error_message: Optional[str] = None
    execution_time: float = 0.0


@dataclass
class TestCase:
    """测试用例类"""
    name: str
    enabled: bool = True
    async_func: Optional[callable] = None
    description: str = ""


class TestRunner:
    """测试运行器"""
    
    def __init__(self, config: TestConfig):
        self.config = config
        self.adapter = None
        self.test_cases: List[TestCase] = []
        self.results: List[TestResult] = []
        self.reply_message_id = ""  # 用于存储回复测试的 message_id
        self.recall_message_id = ""  # 用于存储撤回测试的 message_id
        
    def setup(self):
        """初始化"""
        self.adapter = getattr(sdk.adapter, self.config.adapter_name).Send
        self._register_test_cases()
        
    def _read_file(self, filename: str) -> Optional[bytes]:
        """读取文件内容"""
        file_path = Path(self.config.test_files_dir) / filename
        if not file_path.exists():
            return None
        try:
            with open(file_path, 'rb') as f:
                return f.read()
        except Exception:
            return None
    
    def _register_test_cases(self):
        """注册所有测试用例"""
        # 基础测试
        self._add_basic_tests()
        # 媒体测试
        self._add_media_tests()
        # 高级功能测试
        self._add_advanced_tests()
        # 格式化消息测试
        self._add_format_tests()
        # 链式调用测试
        self._add_chain_tests()
        # @功能测试
        self._add_mention_tests()
    
    def _add_basic_tests(self):
        """添加基础测试用例"""
        self.test_cases.extend([
            TestCase("发送文本消息", self.config.enable_basic_tests, None),
            TestCase("发送@用户消息", self.config.enable_basic_tests, None),
            TestCase("发送表情（emoji）", self.config.enable_basic_tests, None),
            TestCase("发送Markdown消息", self.config.enable_basic_tests, None),
            TestCase("发送Html消息", self.config.enable_basic_tests, None),
        ])
    
    def _add_media_tests(self):
        """添加媒体测试用例"""
        self.test_cases.extend([
            TestCase("发送图片（本地文件）", self.config.enable_media_tests, None),
            TestCase("发送图片（URL）", self.config.enable_media_tests, None),
            TestCase("发送视频（本地文件）", self.config.enable_media_tests, None),
            TestCase("发送视频（URL）", self.config.enable_media_tests, None),
            TestCase("发送语音（本地文件）", self.config.enable_media_tests, None),
            TestCase("发送语音（URL）", self.config.enable_media_tests, None),
        ])
    
    def _add_advanced_tests(self):
        """添加高级功能测试用例"""
        self.test_cases.extend([
            TestCase("发送文件（本地）", self.config.enable_advanced_tests, None),
            TestCase("发送文件（URL）", self.config.enable_advanced_tests, None),
            TestCase("发送回复消息", self.config.enable_advanced_tests, None),
            TestCase("发送组合消息", self.config.enable_advanced_tests, None),
            TestCase("撤回消息", self.config.enable_advanced_tests, None),
        ])
    
    def _add_format_tests(self):
        """添加格式化消息测试用例"""
        self.test_cases.extend([
            TestCase("发送格式化消息（Raw_ob12）", self.config.enable_format_tests, None),
            TestCase("发送文本消息段", self.config.enable_format_tests, None),
            TestCase("发送组合消息段", self.config.enable_format_tests, None),
        ])
    
    def _add_chain_tests(self):
        """添加链式调用测试用例"""
        self.test_cases.extend([
            TestCase("多次@用户（链式调用）", self.config.enable_chain_tests, None),
            TestCase("链式调用 - 回复+@用户", self.config.enable_chain_tests, None),
            TestCase("链式调用 - 组合修饰符", self.config.enable_chain_tests, None),
            TestCase("格式化消息 + 链式@", self.config.enable_chain_tests, None),
            TestCase("复杂组合消息", self.config.enable_chain_tests, None),
        ])
    
    def _add_mention_tests(self):
        """添加@功能测试用例"""
        self.test_cases.extend([
            TestCase("@全体成员", self.config.enable_mention_tests, None),
            TestCase("@全体 + @用户组合", self.config.enable_mention_tests, None),
        ])
    
    def _check_response(self, response: Any) -> tuple[bool, Optional[dict]]:
        """检查响应是否成功"""
        if response is None:
            return False, None
        
        # 如果是 Task 对象，获取结果
        if hasattr(response, '__await__'):
            # 这是一个协程，已经被 await 了
            if isinstance(response, dict):
                resp = response
            else:
                return False, None
        elif isinstance(response, dict):
            resp = response
        else:
            return False, None
        
        # 检查响应格式是否符合标准
        if isinstance(resp, dict):
            status = resp.get("status")
            retcode = resp.get("retcode", -1)
            
            if status == "ok" and retcode == 0:
                return True, resp
        
        return False, resp
    
    async def run_test(self, test_num: int):
        """运行单个测试"""
        if test_num > len(self.test_cases) or test_num < 1:
            return
        
        test_case = self.test_cases[test_num - 1]
        print(f"{test_num}. {test_case.name}")
        
        result = TestResult(
            test_num=test_num,
            test_name=test_case.name,
            status="skipped"
        )
        
        start_time = time.time()
        
        try:
            # 执行测试
            response = await self._execute_test(test_num)
            
            # 检查结果
            success, resp_dict = self._check_response(response)
            
            result.execution_time = time.time() - start_time
            result.response = resp_dict
            
            if success:
                result.status = "success"
                print(f"  成功 - {result.execution_time:.2f}s")
            else:
                result.status = "failed"
                if resp_dict:
                    retcode = resp_dict.get("retcode", -1)
                    message = resp_dict.get("message", "")
                    print(f"  失败 - retcode: {retcode}, message: {message}")
                else:
                    print(f"  失败 - 无效响应")
                    
        except Exception as e:
            result.execution_time = time.time() - start_time
            result.status = "error"
            result.error_message = str(e)
            print(f"  错误 - {type(e).__name__}: {str(e)}")
        
        self.results.append(result)
        await asyncio.sleep(self.config.send_interval)
    
    async def _execute_test(self, test_num: int) -> Optional[Any]:
        """执行具体的测试逻辑"""
        group_id = self.config.group_id
        test_user_id = self.config.test_user_id
        
        # 1. 发送文本消息
        if test_num == 1:
            return await self.adapter.To("group", group_id).Text("Hello, 这是一条测试消息！")
        
        # 2. 发送@用户消息
        elif test_num == 2:
            return await self.adapter.To("group", group_id).At(test_user_id).Text("@某位成员")
        
        # 3. 发送表情
        elif test_num == 3:
            return await self.adapter.To("group", group_id).Face("1")
        
        # 4. 发送Markdown消息
        elif test_num == 4:
            markdown_text = "**粗体** 和 *斜体* 文本测试"
            return await self.adapter.To("group", group_id).Markdown(markdown_text)
        
        # 5. 发送Html消息
        elif test_num == 5:
            html_text = "<b>粗体</b> 和 <i>斜体</i> 文本测试"
            return await self.adapter.To("group", group_id).Html(html_text)
        
        # 6. 发送图片（本地文件）
        elif test_num == 6:
            image_data = self._read_file(self.config.image_file)
            if image_data:
                return await self.adapter.To("group", group_id).Image(image_data)
            # 回退到 URL 方式
            return await self.adapter.To("group", group_id).Image(self.config.image_url)
        
        # 7. 发送图片（URL）
        elif test_num == 7:
            return await self.adapter.To("group", group_id).Image(self.config.image_url)
        
        # 8. 发送视频（本地文件）
        elif test_num == 8:
            video_data = self._read_file(self.config.video_file)
            if video_data:
                return await self.adapter.To("group", group_id).Video(video_data)
            # 回退到 URL 方式
            return await self.adapter.To("group", group_id).Video(self.config.video_url)
        
        # 9. 发送视频（URL）
        elif test_num == 9:
            return await self.adapter.To("group", group_id).Video(self.config.video_url)
        
        # 10. 发送语音（本地文件）
        elif test_num == 10:
            if self.config.voice_file:
                voice_data = self._read_file(self.config.voice_file)
                if voice_data:
                    return await self.adapter.To("group", group_id).Voice(voice_data)
            # 回退到 URL 方式
            return await self.adapter.To("group", group_id).Voice(self.config.voice_url)
        
        # 11. 发送语音（URL）
        elif test_num == 11:
            return await self.adapter.To("group", group_id).Voice(self.config.voice_url)
        
        # 12. 发送文件（本地）
        elif test_num == 12:
            file_data = self._read_file(self.config.doc_file)
            if file_data:
                return await self.adapter.To("group", group_id).File(file_data)
            return await self.adapter.To("group", group_id).File(self.config.file_url)
        
        # 13. 发送文件（URL）
        elif test_num == 13:
            return await self.adapter.To("group", group_id).File(self.config.file_url)
        
        # 14. 发送回复消息
        elif test_num == 14:
            test_message = "这是一条测试消息，用于后续回复功能测试"
            result = await self.adapter.To("group", group_id).Text(test_message)
            
            # 尝试获取 message_id
            if isinstance(result, dict) and result.get("data", {}).get("message_id"):
                self.reply_message_id = result["data"]["message_id"]
            else:
                self.reply_message_id = "temp_msg_id_" + str(int(time.time()))
            
            return result
        
        # 15. 发送组合消息
        elif test_num == 15:
            ob12_message = [
                {"type": "text", "data": {"text": "组合消息测试："}},
                {"type": "mention", "data": {"user_id": test_user_id}}
            ]
            return await self.adapter.To("group", group_id).Raw_ob12(ob12_message)
        
        # 16. 撤回消息
        elif test_num == 16:
            # 先发送一条消息
            test_message = "这条消息将被撤回"
            result = await self.adapter.To("group", group_id).Text(test_message)
            
            print(f"发送消息：{result}")
            # 获取 message_id
            self.recall_message_id = result.get("data", {}).get("message_id")
            
            # 等待一下再撤回
            await asyncio.sleep(2)
            
            print(f"撤回消息：{self.recall_message_id}")

            # 撤回消息
            return await self.adapter.To("group", group_id).Recall(self.recall_message_id)
        
        # 17. 发送格式化消息（Raw_ob12）
        elif test_num == 17:
            ob12_message = [
                {"type": "text", "data": {"text": "这是格式化消息 "}},
                {"type": "text", "data": {"text": "使用 Raw_ob12 发送"}}
            ]
            return await self.adapter.To("group", group_id).Raw_ob12(ob12_message)
        
        # 18. 发送文本消息段
        elif test_num == 18:
            ob12_message = [
                {"type": "text", "data": {"text": "第一条文本消息段"}},
                {"type": "text", "data": {"text": "第二条文本消息段"}}
            ]
            return await self.adapter.To("group", group_id).Raw_ob12(ob12_message)
        
        # 19. 发送组合消息段
        elif test_num == 19:
            ob12_message = [
                {"type": "text", "data": {"text": "文本 + 图片："}},
                {"type": "image", "data": {"file": self.config.image_url}}
            ]
            return await self.adapter.To("group", group_id).Raw_ob12(ob12_message)
        
        # 20. 多次@用户（链式调用）
        elif test_num == 20:
            return await self.adapter.To("group", group_id).At(test_user_id).At(self.config.test_user_id_2).Text(" @多个用户")
        
        # 21. 链式调用 - 回复+@用户
        elif test_num == 21:
            return await self.adapter.To("group", group_id).Reply(self.reply_message_id).At(test_user_id).Text("回复并@用户")
        
        # 22. 链式调用 - 组合修饰符
        elif test_num == 22:
            return await self.adapter.To("group", group_id).At(test_user_id).Reply(self.reply_message_id).Text("@用户并回复")
        
        # 23. 格式化消息 + 链式@
        elif test_num == 23:
            ob12_message = [{"type": "text", "data": {"text": "格式化消息 + 链式@"}}]
            return await self.adapter.To("group", group_id).At(test_user_id).Raw_ob12(ob12_message)
        
        # 24. 复杂组合消息
        elif test_num == 24:
            ob12_message = [
                {"type": "text", "data": {"text": "复杂组合消息："}},
                {"type": "mention", "data": {"user_id": test_user_id}},
                {"type": "reply", "data": {"message_id": self.reply_message_id}}
            ]
            return await self.adapter.To("group", group_id).Raw_ob12(ob12_message)
        
        # 25. @全体成员
        elif test_num == 25:
            return await self.adapter.To("group", group_id).AtAll().Text("这是全体成员消息")
        
        # 26. @全体 + @用户组合
        elif test_num == 26:
            return await self.adapter.To("group", group_id).AtAll().At(test_user_id).Text("全体 + 单个@")
        
        return None
    
    def _print_summary(self):
        """打印测试结果汇总"""
        success_count = sum(1 for r in self.results if r.status == "success")
        failed_count = sum(1 for r in self.results if r.status == "failed")
        error_count = sum(1 for r in self.results if r.status == "error")
        total_time = sum(r.execution_time for r in self.results)
        
        print("\n___")
        print("测试结果")
        print("    总结")
        print(f"         成功：{success_count}个")
        print(f"         失败：{failed_count}个")
        print(f"         错误：{error_count}个")
        
        if failed_count > 0:
            print("\n    失败详情")
            for r in self.results:
                if r.status == "failed":
                    if r.response:
                        retcode = r.response.get("retcode", -1)
                        message = r.response.get("message", "")
                        print(f"         [{r.test_num}] {r.test_name} - retcode: {retcode}, message: {message}")
                    else:
                        print(f"         [{r.test_num}] {r.test_name} - 无响应")
        
        if error_count > 0:
            print("\n    错误详情")
            for r in self.results:
                if r.status == "error":
                    print(f"         [{r.test_num}] {r.test_name} - {r.error_message}")
        
        print(f"\n    执行时间：{total_time:.2f} 秒")
        print("___")
    
    async def run_all(self):
        """运行所有启用的测试"""
        enabled_tests = []
        
        # 如果指定了特定测试，只运行这些测试
        if self.config.specific_tests:
            for test_num in self.config.specific_tests:
                if 1 <= test_num <= len(self.test_cases):
                    enabled_tests.append(test_num)
                else:
                    print(f"[警告] 无效的测试编号: {test_num}")
        else:
            # 否则根据配置运行所有启用的测试
            for i, test_case in enumerate(self.test_cases, 1):
                if test_case.enabled:
                    enabled_tests.append(i)
        
        print(f"准备运行 {len(enabled_tests)} 个测试用例")
        print("=" * 50)
        
        for test_num in enabled_tests:
            await self.run_test(test_num)
        
        print("=" * 50)
        self._print_summary()


async def main():
    try:
        isInit = await sdk.init_task()
        
        if not isInit:
            sdk.logger.error("ErisPulse 初始化失败，请检查日志")
            return
        
        await sdk.adapter.startup()

        # 等待适配器完全启动
        await asyncio.sleep(3)
        
        # 创建测试配置
        config = TestConfig()
        
        # 配置示例：
        # 1. 指定适配器
        # config.adapter_name = "onebot12"
        # config.adapter_name = "red"
        
        # 2. 只运行特定测试
        # config.specific_tests = [6, 7, 8, 9, 10, 11]  # 只运行媒体测试
        
        # 3. 禁用某些测试类别
        # config.enable_media_tests = False  # 禁用媒体测试
        # config.enable_chain_tests = False   # 禁用链式调用测试
        
        # 4. 配置文件路径
        # config.video_file = "my_video.mp4"
        # config.voice_file = "my_voice.amr"
        # config.test_files_dir = "custom/files"
        
        # 配置发送间隔（秒）
        config.send_interval = 1.0
        
        print(f"测试适配器: {config.adapter_name}")
        print(f"测试目标: 群号 {config.group_id}")
        print("=" * 50)
        
        # 创建并运行测试
        runner = TestRunner(config)
        runner.setup()
        await runner.run_all()
        
        # 保持程序运行(不建议修改)
        await asyncio.Event().wait()
    except Exception as e:
        sdk.logger.error(f"发生错误: {e}", exc_info=True)
    except KeyboardInterrupt:
        sdk.logger.info("正在停止程序")
    finally:
        await sdk.adapter.shutdown()


if __name__ == "__main__":
    asyncio.run(main())



from typing import Dict, Any, Optional


class ResponseFormatter:
    PLATFORM_NAME = "yunhu_user"
    
    @staticmethod
    def success(
        message_id: str = "",
        data: Optional[Dict] = None,
        raw_response: Optional[Dict] = None
    ) -> Dict[str, Any]:
        """
        生成成功响应
        
        :param message_id: 消息 ID
        :param data: 响应数据
        :param raw_response: 原始平台响应
        :return: OneBot12 标准响应
        """
        result_data = data
        if message_id and not result_data:
            result_data = {"message_id": message_id}
        elif message_id and result_data is None:
            result_data = {"message_id": message_id}
        elif not result_data and message_id:
            result_data = {"message_id": message_id}
        
        return {
            "status": "ok",
            "retcode": 0,
            "data": result_data or {},
            "message_id": message_id,
            "message": "",
            f"{ResponseFormatter.PLATFORM_NAME}_raw": raw_response or {}
        }
    
    @staticmethod
    def failed(
        code: int,
        message: str,
        raw_response: Optional[Dict] = None
    ) -> Dict[str, Any]:
        """
        生成失败响应
        
        :param code: 返回码（遵循 OneBot12 规范）
        :param message: 错误信息
        :param raw_response: 原始平台响应
        :return: OneBot12 标准响应
        """
        return {
            "status": "failed",
            "retcode": code,
            "data": None,
            "message_id": "",
            "message": message,
            f"{ResponseFormatter.PLATFORM_NAME}_raw": raw_response or {}
        }
    
    @staticmethod
    def from_platform_response(
        response: Dict[str, Any],
        fallback_message_id: str = ""
    ) -> Dict[str, Any]:
        """
        从云湖平台响应转换为 OneBot12 标准格式
        
        :param response: 云湖平台响应
        :param fallback_message_id: 备用消息 ID
        :return: OneBot12 标准响应
        """
        # 检查响应有效性
        if not response or not isinstance(response, dict):
            return ResponseFormatter.failed(
                34000,
                "无效的响应格式",
                str(response)
            )
        
        # 检查 status 字段
        status = response.get("status")
        if not status or not isinstance(status, dict):
            return ResponseFormatter.failed(
                34000,
                "响应缺少status字段",
                response
            )
        
        # 检查 code 字段
        code = status.get("code")
        if code is None:
            return ResponseFormatter.failed(
                34000,
                "响应缺少code字段",
                response
            )
        
        # 处理成功情况
        if code == 1:
            data = response.get("data", {})
            message_id = ""
            
            # 尝试从 data 中获取 message_id
            if data and isinstance(data, dict):
                message_id = data.get("message_id", "")
            elif isinstance(data, str):
                message_id = data
            
            # 优先使用 API 返回的 message_id，否则使用 fallback
            final_message_id = message_id or fallback_message_id
            
            return ResponseFormatter.success(
                message_id=final_message_id,
                data={"message_id": final_message_id} if final_message_id else {},
                raw_response=response
            )
        
        # 处理失败情况
        else:
            error_msg = status.get("msg", "未知错误")
            return ResponseFormatter.failed(
                10003,
                error_msg,
                response
            )
    
    @staticmethod
    def error(error: Exception) -> Dict[str, Any]:
        """
        生成异常响应
        
        :param error: 异常对象
        :return: OneBot12 标准响应
        """
        return ResponseFormatter.failed(
            34000,
            str(error),
            str(error)
        )


class MessageTypes:
    TEXT = 1
    IMAGE = 2
    MARKDOWN = 3
    FILE = 4
    FORM = 5
    ARTICLE = 6
    STICKER = 7
    HTML = 8
    AUDIO = 11
    VIDEO = 10


class ChatType:
    USER = 1
    PRIVATE = 1
    GROUP = 2
    BOT = 3


# 会话类型映射（OneBot12 detail_type -> 云湖 chat_type）
CHAT_TYPE_MAP = {
    "user": ChatType.USER,
    "private": ChatType.PRIVATE,
    "group": ChatType.GROUP,
    "bot": ChatType.BOT
}

# 内容类型映射（content_type -> 云湖 msg_type）
CONTENT_TYPE_MAP = {
    "text": MessageTypes.TEXT,
    "markdown": MessageTypes.MARKDOWN,
    "html": MessageTypes.HTML
}


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: ws.proto
# Protobuf Python Version: 6.33.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    5,
    '',
    'ws.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x08ws.proto\x12\x08yh_ws_go\" \n\x04INFO\x12\x0b\n\x03seq\x18\x01 \x01(\t\x12\x0b\n\x03\x63md\x18\x02 \x01(\t\".\n\x03Tag\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0c\n\x04text\x18\x03 \x01(\t\x12\r\n\x05\x63olor\x18\x04 \x01(\t\"\x97\x08\n\x03Msg\x12\x0e\n\x06msg_id\x18\x01 \x01(\t\x12$\n\x06sender\x18\x02 \x01(\x0b\x32\x14.yh_ws_go.Msg.Sender\x12\x0f\n\x07recv_id\x18\x03 \x01(\t\x12\x0f\n\x07\x63hat_id\x18\x04 \x01(\t\x12\x11\n\tchat_type\x18\x05 \x01(\x05\x12&\n\x07\x63ontent\x18\x06 \x01(\x0b\x32\x15.yh_ws_go.Msg.Content\x12\x14\n\x0c\x63ontent_type\x18\x07 \x01(\x05\x12\x11\n\ttimestamp\x18\x08 \x01(\x03\x12\x1e\n\x03\x63md\x18\t \x01(\x0b\x32\x11.yh_ws_go.Msg.Cmd\x12\x13\n\x0b\x64\x65lete_time\x18\n \x01(\x03\x12\x14\n\x0cquote_msg_id\x18\x0b \x01(\t\x12\x0f\n\x07msg_seq\x18\x0c \x01(\x03\x12\x11\n\tedit_time\x18\x0e \x01(\x03\x1a\x1f\n\x03\x43md\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0c\n\x04name\x18\x02 \x01(\t\x1a{\n\x06Sender\x12\x0f\n\x07\x63hat_id\x18\x01 \x01(\t\x12\x11\n\tchat_type\x18\x02 \x01(\x05\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x12\n\navatar_url\x18\x04 \x01(\t\x12\x0f\n\x07tag_old\x18\x06 \x03(\t\x12\x1a\n\x03tag\x18\x07 \x03(\x0b\x32\r.yh_ws_go.Tag\x1a\xc6\x04\n\x07\x43ontent\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x0f\n\x07\x62uttons\x18\x02 \x01(\t\x12\x11\n\timage_url\x18\x03 \x01(\t\x12\x11\n\tfile_name\x18\x04 \x01(\t\x12\x10\n\x08\x66ile_url\x18\x05 \x01(\t\x12\x0c\n\x04\x66orm\x18\x07 \x01(\t\x12\x16\n\x0equote_msg_text\x18\x08 \x01(\t\x12\x13\n\x0bsticker_url\x18\t \x01(\t\x12\x0f\n\x07post_id\x18\n \x01(\t\x12\x12\n\npost_title\x18\x0b \x01(\t\x12\x14\n\x0cpost_content\x18\x0c \x01(\t\x12\x19\n\x11post_content_type\x18\r \x01(\t\x12\x15\n\rexpression_id\x18\x0f \x01(\t\x12\x17\n\x0fquote_image_url\x18\x10 \x01(\t\x12\x18\n\x10quote_image_name\x18\x11 \x01(\t\x12\x11\n\tfile_size\x18\x12 \x01(\x03\x12\x11\n\tvideo_url\x18\x13 \x01(\t\x12\x11\n\taudio_url\x18\x15 \x01(\t\x12\x12\n\naudio_time\x18\x16 \x01(\x03\x12\x17\n\x0fquote_video_url\x18\x17 \x01(\t\x12\x18\n\x10quote_video_time\x18\x18 \x01(\x03\x12\x17\n\x0fsticker_item_id\x18\x19 \x01(\x03\x12\x17\n\x0fsticker_pack_id\x18\x1a \x01(\x03\x12\x11\n\tcall_text\x18\x1d \x01(\t\x12\x18\n\x10\x63\x61ll_status_text\x18  \x01(\t\x12\r\n\x05width\x18! \x01(\x03\x12\x0e\n\x06height\x18\" \x01(\x03\x12\x0b\n\x03tip\x18% \x01(\t\"-\n\rheartbeat_ack\x12\x1c\n\x04info\x18\x01 \x01(\x0b\x32\x0e.yh_ws_go.INFO\"\x88\x01\n\x0cpush_message\x12\x1c\n\x04info\x18\x01 \x01(\x0b\x32\x0e.yh_ws_go.INFO\x12)\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x1b.yh_ws_go.push_message.Data\x1a/\n\x04\x44\x61ta\x12\x0b\n\x03\x61ny\x18\x01 \x01(\t\x12\x1a\n\x03msg\x18\x02 \x01(\x0b\x32\r.yh_ws_go.Msg\"\x88\x01\n\x0c\x65\x64it_message\x12\x1c\n\x04info\x18\x01 \x01(\x0b\x32\x0e.yh_ws_go.INFO\x12)\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x1b.yh_ws_go.edit_message.Data\x1a/\n\x04\x44\x61ta\x12\x0b\n\x03\x61ny\x18\x01 \x01(\t\x12\x1a\n\x03msg\x18\x02 \x01(\x0b\x32\r.yh_ws_go.Msg\"\xac\x02\n\x11\x66ile_send_message\x12\x1c\n\x04info\x18\x01 \x01(\x0b\x32\x0e.yh_ws_go.INFO\x12.\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32 .yh_ws_go.file_send_message.Data\x1a\xc8\x01\n\x04\x44\x61ta\x12\x0b\n\x03\x61ny\x18\x01 \x01(\t\x12\x37\n\x06sender\x18\x02 \x01(\x0b\x32\'.yh_ws_go.file_send_message.Data.Sender\x1az\n\x06Sender\x12\x14\n\x0csend_user_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x11\n\ttemp_code\x18\x03 \x01(\x04\x12\x11\n\tsend_type\x18\x04 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\t\x12\x15\n\rsend_deviceId\x18\x06 \x01(\t\"\xc4\x01\n\x0b\x64raft_input\x12\x1c\n\x04info\x18\x01 \x01(\x0b\x32\x0e.yh_ws_go.INFO\x12(\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x1a.yh_ws_go.draft_input.Data\x1am\n\x04\x44\x61ta\x12\x0b\n\x03\x61ny\x18\x01 \x01(\t\x12/\n\x05\x64raft\x18\x02 \x01(\x0b\x32 .yh_ws_go.draft_input.Data.Draft\x1a\'\n\x05\x44raft\x12\x0f\n\x07\x63hat_id\x18\x01 \x01(\t\x12\r\n\x05input\x18\x02 \x01(\t\"\xc9\x02\n\x11\x62ot_board_message\x12\x1c\n\x04info\x18\x01 \x01(\x0b\x32\x0e.yh_ws_go.INFO\x12.\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32 .yh_ws_go.bot_board_message.Data\x1a\xe5\x01\n\x04\x44\x61ta\x12\x0b\n\x03\x61ny\x18\x01 \x01(\t\x12:\n\x05\x62oard\x18\x02 \x01(\x0b\x32+.yh_ws_go.bot_board_message.Data.Board_data\x1a\x93\x01\n\nBoard_data\x12\x0e\n\x06\x62ot_id\x18\x01 \x01(\t\x12\x0f\n\x07\x63hat_id\x18\x02 \x01(\t\x12\x11\n\tchat_type\x18\x03 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x04 \x01(\t\x12\x14\n\x0c\x63ontent_type\x18\x05 \x01(\x05\x12\x18\n\x10last_update_time\x18\x06 \x01(\x03\x12\x10\n\x08\x62ot_name\x18\x07 \x01(\t\"\xf7\x01\n\x0estream_message\x12\x1c\n\x04info\x18\x01 \x01(\x0b\x32\x0e.yh_ws_go.INFO\x12+\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x1d.yh_ws_go.stream_message.Data\x1a\x99\x01\n\x04\x44\x61ta\x12\x0b\n\x03\x61ny\x18\x01 \x01(\t\x12\x34\n\x03msg\x18\x02 \x01(\x0b\x32\'.yh_ws_go.stream_message.Data.StreamMsg\x1aN\n\tStreamMsg\x12\x0e\n\x06msg_id\x18\x01 \x01(\t\x12\x0f\n\x07recv_id\x18\x02 \x01(\t\x12\x0f\n\x07\x63hat_id\x18\x03 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x04 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ws_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_INFO']._serialized_start=22
  _globals['_INFO']._serialized_end=54
  _globals['_TAG']._serialized_start=56
  _globals['_TAG']._serialized_end=102
  _globals['_MSG']._serialized_start=105
  _globals['_MSG']._serialized_end=1152
  _globals['_MSG_CMD']._serialized_start=411
  _globals['_MSG_CMD']._serialized_end=442
  _globals['_MSG_SENDER']._serialized_start=444
  _globals['_MSG_SENDER']._serialized_end=567
  _globals['_MSG_CONTENT']._serialized_start=570
  _globals['_MSG_CONTENT']._serialized_end=1152
  _globals['_HEARTBEAT_ACK']._serialized_start=1154
  _globals['_HEARTBEAT_ACK']._serialized_end=1199
  _globals['_PUSH_MESSAGE']._serialized_start=1202
  _globals['_PUSH_MESSAGE']._serialized_end=1338
  _globals['_PUSH_MESSAGE_DATA']._serialized_start=1291
  _globals['_PUSH_MESSAGE_DATA']._serialized_end=1338
  _globals['_EDIT_MESSAGE']._serialized_start=1341
  _globals['_EDIT_MESSAGE']._serialized_end=1477
  _globals['_EDIT_MESSAGE_DATA']._serialized_start=1291
  _globals['_EDIT_MESSAGE_DATA']._serialized_end=1338
  _globals['_FILE_SEND_MESSAGE']._serialized_start=1480
  _globals['_FILE_SEND_MESSAGE']._serialized_end=1780
  _globals['_FILE_SEND_MESSAGE_DATA']._serialized_start=1580
  _globals['_FILE_SEND_MESSAGE_DATA']._serialized_end=1780
  _globals['_FILE_SEND_MESSAGE_DATA_SENDER']._serialized_start=1658
  _globals['_FILE_SEND_MESSAGE_DATA_SENDER']._serialized_end=1780
  _globals['_DRAFT_INPUT']._serialized_start=1783
  _globals['_DRAFT_INPUT']._serialized_end=1979
  _globals['_DRAFT_INPUT_DATA']._serialized_start=1870
  _globals['_DRAFT_INPUT_DATA']._serialized_end=1979
  _globals['_DRAFT_INPUT_DATA_DRAFT']._serialized_start=1940
  _globals['_DRAFT_INPUT_DATA_DRAFT']._serialized_end=1979
  _globals['_BOT_BOARD_MESSAGE']._serialized_start=1982
  _globals['_BOT_BOARD_MESSAGE']._serialized_end=2311
  _globals['_BOT_BOARD_MESSAGE_DATA']._serialized_start=2082
  _globals['_BOT_BOARD_MESSAGE_DATA']._serialized_end=2311
  _globals['_BOT_BOARD_MESSAGE_DATA_BOARD_DATA']._serialized_start=2164
  _globals['_BOT_BOARD_MESSAGE_DATA_BOARD_DATA']._serialized_end=2311
  _globals['_STREAM_MESSAGE']._serialized_start=2314
  _globals['_STREAM_MESSAGE']._serialized_end=2561
  _globals['_STREAM_MESSAGE_DATA']._serialized_start=2408
  _globals['_STREAM_MESSAGE_DATA']._serialized_end=2561
  _globals['_STREAM_MESSAGE_DATA_STREAMMSG']._serialized_start=2483
  _globals['_STREAM_MESSAGE_DATA_STREAMMSG']._serialized_end=2561
# @@protoc_insertion_point(module_scope)



"""
云湖用户账户驱动适配器核心类

支持通过用户邮箱账户与云湖平台交互，使用 WebSocket 接收事件
"""

import asyncio
import uuid
from typing import Dict, Any, Optional
from dataclasses import dataclass

# ErisPulse 核心导入
from ErisPulse.Core import logger, config as config_manager, adapter
from ErisPulse.Core.Bases import BaseAdapter

# 本地导入
from .client.http import YunhuHTTPClient
from .client.ws import YunhuWSClient
from .Send import Send
from .utils.constants import CHAT_TYPE_MAP
from .utils.response_formatter import ResponseFormatter


@dataclass
class AccountConfig:
    """账户配置"""
    email: str
    password: str
    platform: str = "windows"
    device_id: str = ""
    enabled: bool = True


class YunhuUserAdapter(BaseAdapter):
    """
    云湖用户账户驱动适配器
    
    支持通过用户邮箱账户登录，使用 WebSocket 接收消息事件
    """
    
    Send = Send

    def __init__(self, sdk):
        
        # 调用父类初始化（必须，以初始化 self.Send）
        super().__init__()
        
        self.logger = logger
        self.config_manager = config_manager
        self.adapter = adapter
        
        # 账户配置
        self.accounts = {}
        self._account_configs = {}
        
        # HTTP 和 WebSocket 客户端
        self._http_clients = {}
        self._ws_clients = {}
        self._ws_tasks = {}
        self._running = False
        
        # 转换器
        self.converter = None
        
        # 默认配置
        self.config = self._get_config()
        
        self.logger.info("YunhuUserAdapter 初始化完成")
    
    def _get_config(self) -> Dict[str, Any]:
        """
        获取适配器配置
        
        :return: 配置字典
        """
        if not self.config_manager:
            return {}
        
        config = self.config_manager.getConfig("YunhuUserAdapter", {})
        
        if config is None:
            default_config = {
                "ws_reconnect_interval": 30,
                "ws_timeout": 70
            }
            self.config_manager.setConfig("YunhuUserAdapter", default_config)
            return default_config
        
        # 确保配置值为正确的类型
        if "ws_reconnect_interval" in config:
            val = config["ws_reconnect_interval"]
            if isinstance(val, str):
                config["ws_reconnect_interval"] = int(val)
        
        if "ws_timeout" in config:
            val = config["ws_timeout"]
            if isinstance(val, str):
                config["ws_timeout"] = int(val)
        
        return config
    
    def _load_accounts(self) -> None:
        """
        加载账户配置
        
        从配置文件中读取账户信息
        """
        self._account_configs.clear()
        
        # 读取账户配置
        full_config = self.config_manager.getConfig("YunhuUserAdapter", {})
        
        self.logger.debug(f"全量配置: {full_config}")

        # 支持嵌套结构：{'accounts': {'default': {...}}}
        accounts_config = full_config.get("accounts", {})
        
        for key, value in full_config.items():
            if key.startswith("accounts.") and isinstance(value, dict):
                accounts_config[key.split(".", 1)[1]] = value
        
        for account_name, value in accounts_config.items():
            if not isinstance(value, dict):
                continue
                
            # 检查是否启用
            if value.get("enabled", True):
                account_config = AccountConfig(
                    email=value.get("email", ""),
                    password=value.get("password", ""),
                    platform=value.get("platform", "windows"),
                    device_id=value.get("device_id", ""),
                    enabled=value.get("enabled", True)
                )
                
                if account_config.email and account_config.password:
                    self._account_configs[account_name] = account_config
                    self.logger.info(f"加载账户: {account_name} ({account_config.email})")
    
    async def _login_account(self, account_name: str, account_config: AccountConfig) -> Optional[str]:
        """
        登录账户并获取 token 和 user_id
        
        :param account_name: 账户名称
        :param account_config: 账户配置
        :return: 用户 ID，失败返回 None
        """
        http_client = None
        try:
            self.logger.info(f"正在登录账户: {account_name} ({account_config.email})")
            
            # 创建 HTTP 客户端
            http_client = YunhuHTTPClient()
            
            # 登录
            login_response = await http_client.email_login(
                email=account_config.email,
                password=account_config.password,
                device_id=account_config.device_id or uuid.uuid4().hex,
                platform=account_config.platform
            )
            
            self.logger.debug(f"登录响应: {login_response}")

            # 检查登录结果
            if not login_response or login_response.get("code") != 1:
                error_msg = login_response.get("msg", "登录失败") if login_response else "登录失败"
                self.logger.error(f"账户 {account_name} 登录失败: {error_msg}")
                return None
            
            # 获取 token
            token = login_response.get("data", {}).get("token", "")
            if not token:
                self.logger.error(f"账户 {account_name} 未获取到 token")
                return None
            
            # 获取用户信息
            http_client.set_token(token)
            user_info_response = await http_client.get_user_info()

            self.logger.debug(f"用户信息响应: {user_info_response}")
            
            # 获取用户 ID，直接检查 data 是否存在
            data = user_info_response.get("data")
            user_id = data.id if data else ""
            
            # 检查是否成功获取到用户 ID
            if not user_id:
                self.logger.error(f"账户 {account_name} 获取用户信息失败：未获取到用户 ID")
                return None
            
            self.logger.info(f"账户 {account_name} 登录成功，用户ID: {user_id}")
            
            # 保存账户信息
            self.accounts[account_name] = {
                "name": account_name,
                "email": account_config.email,
                "token": token,
                "user_id": user_id,
                "platform": account_config.platform,
                "device_id": account_config.device_id or uuid.uuid4().hex
            }
            
            # 保存 HTTP 客户端
            self._http_clients[account_name] = http_client
            
            return user_id
            
        except Exception as e:
            self.logger.error(f"账户 {account_name} 登录异常: {e}")
            import traceback
            self.logger.debug(traceback.format_exc())
            return None
        finally:
            # 登录失败时关闭 HTTP 客户端
            if http_client and account_name not in self._http_clients:
                await http_client.close()
    
    async def _handle_ws_event(self, ws_event: Dict[str, Any], account_name: str, account: Dict[str, Any]) -> None:
        """
        处理 WebSocket 事件
        
        :param ws_event: WebSocket 原始事件
        :param account_name: 账户名称
        :param account: 账户信息
        """
        try:
            # 转换事件为 OneBot12 格式
            ob12_event = self.converter.convert(ws_event)
            
            if ob12_event:
                self.logger.debug(f"ob12 事件: {ob12_event}")
                # 填充当前登录用户的 ID
                ob12_event["self"]["user_id"] = account["user_id"]
                
                # 提交事件到适配器系统
                if self.adapter:
                    await self.adapter.emit(ob12_event)
                else:
                    self.logger.warning(f"未设置 adapter，无法提交事件: {ob12_event.get('type')}")
        
        except Exception as e:
            self.logger.error(f"账户 {account_name} 处理 WebSocket 事件失败: {e}")
            import traceback
            self.logger.debug(traceback.format_exc())
    
    async def _ws_listener(self, account_name: str, account: Dict[str, Any]) -> None:
        """
        WebSocket 事件监听器
        
        :param account_name: 账户名称
        :param account: 账户信息
        """
        retry_count = 0
        max_retries = 3
        reconnect_interval = int(self.config.get("ws_reconnect_interval", 30))
        ws_timeout = int(self.config.get("ws_timeout", 70))
        
        while self._running and retry_count < max_retries:
            try:
                self.logger.info(f"账户 {account_name} 正在连接 WebSocket... (尝试 {retry_count + 1}/{max_retries})")
                
                # 创建 WebSocket 客户端
                ws_client = YunhuWSClient(
                    token=account["token"],
                    user_id=account["user_id"],
                    platform=account["platform"],
                    device_id=account["device_id"],
                    timeout=ws_timeout,
                    decode=True,
                    mode="black",
                    blacklist=["heartbeat_ack"]
                )
                
                self._ws_clients[account_name] = ws_client
                
                # 连接并监听事件
                async for ws_event in ws_client.connect():
                    if not self._running:
                        break
                    
                    asyncio.create_task(
                        self._handle_ws_event(ws_event, account_name, account)
                    )
                
                # 正常退出循环
                break
                
            except Exception as e:
                retry_count += 1
                self.logger.error(f"账户 {account_name} WebSocket 连接异常 (重试 {retry_count}/{max_retries}): {e}")
                
                if retry_count < max_retries:
                    # 等待重连
                    await asyncio.sleep(reconnect_interval)
                else:
                    self.logger.error(f"账户 {account_name} 达到最大重试次数，停止重连")
                    break
            
            finally:
                # 清理连接
                if account_name in self._ws_clients:
                    self._ws_clients[account_name] = None
                    del self._ws_clients[account_name]
    
    async def start(self) -> None:
        """
        启动适配器
        
        登录所有配置的账户并建立 WebSocket 连接
        """
        if self._running:
            self.logger.warning("适配器已在运行")
            return
        
        self._running = True
        
        # 初始化转换器
        from .Converter import YunhuUserConverter
        self.converter = YunhuUserConverter()
        
        # 加载账户配置
        self._load_accounts()
        
        if not self._account_configs:
            self.logger.warning("没有找到启用的账户配置")
            self._running = False
            return
        
        self.logger.info(f"找到 {len(self._account_configs)} 个启用的账户")
        
        # 登录所有账户
        for account_name, account_config in self._account_configs.items():
            user_id = await self._login_account(account_name, account_config)
            
            if user_id:
                # 启动 WebSocket 监听任务
                account = self.accounts.get(account_name)
                if account:
                    task = asyncio.create_task(self._ws_listener(account_name, account))
                    self._ws_tasks[account_name] = task
                    self.logger.info(f"账户 {account_name} WebSocket 监听任务已启动")
        
        self.logger.info("YunhuUserAdapter 已启动")
    
    async def shutdown(self) -> None:
        """
        关闭适配器
        
        清理所有资源和连接
        """
        self.logger.info("正在关闭 YunhuUserAdapter...")
        
        self._running = False
        
        # 关闭所有 WebSocket 连接
        for account_name, ws_client in list(self._ws_clients.items()):
            try:
                if ws_client:
                    await ws_client.close()
                self.logger.info(f"账户 {account_name} WebSocket 已关闭")
            except Exception as e:
                self.logger.error(f"关闭账户 {account_name} WebSocket 时出错: {e}")
        
        # 关闭所有 HTTP 客户端
        for account_name, http_client in list(self._http_clients.items()):
            try:
                if http_client:
                    await http_client.close()
            except Exception as e:
                self.logger.error(f"关闭账户 {account_name} HTTP 客户端时出错: {e}")
        
        # 取消所有任务
        for account_name, task in list(self._ws_tasks.items()):
            if task and not task.done():
                task.cancel()
                try:
                    await task
                except asyncio.CancelledError:
                    pass
                self.logger.info(f"账户 {account_name} 任务已取消")
        
        # 清理资源
        self._ws_clients.clear()
        self._http_clients.clear()
        self._ws_tasks.clear()
        self.accounts.clear()
        
        self.logger.info("YunhuUserAdapter 已关闭")
    
    def _get_account_by_user_id(self, user_id: str) -> Optional[str]:
        """
        根据 user_id 查找账户
        
        :param user_id: 用户 ID
        :return: 账户名称
        """
        for account_name, account in self.accounts.items():
            if account.get("user_id") == user_id:
                return account_name
        return None
    
    def _get_account_by_id(self, account_id: str) -> Optional[Dict[str, Any]]:
        """
        根据账户 ID（名称或 user_id）查找账户
        
        :param account_id: 账户 ID
        :return: 账户信息
        """
        # 直接查找账户名称
        if account_id in self.accounts:
            return self.accounts[account_id]
        
        # 根据 user_id 查找
        return next((acc for acc in self.accounts.values() if acc.get("user_id") == account_id), None)
    
    def _get_http_client(self, account_name: str) -> Optional[YunhuHTTPClient]:
        """获取指定账户的 HTTP 客户端"""
        return self._http_clients.get(account_name)

    async def call_api(self, endpoint: str, **params) -> Dict[str, Any]:
        """
        调用平台 API

        :param endpoint: API 端点
        :param params: 参数
        :return: OneBot12 标准响应
        """
        try:
            # 获取账户
            account_id = params.get("account_id")
            account = self._get_account_by_id(account_id) if account_id else None
            if not account:
                # 使用第一个启用的账户
                if self.accounts:
                    account = next(iter(self.accounts.values()))
                else:
                    return {
                        "status": "failed",
                        "retcode": 10003,
                        "data": None,
                        "message_id": "",
                        "message": "没有可用的账户",
                        "yunhu_user_raw": {}
                    }

            # 获取 HTTP 客户端
            account_name = account["name"]
            http_client = self._http_clients.get(account_name)
            if not http_client:
                return {
                    "status": "failed",
                    "retcode": 10003,
                    "data": None,
                    "message_id": "",
                    "message": "HTTP 客户端未初始化",
                    "yunhu_user_raw": {}
                }

            # 根据端点分发处理
            if endpoint == "/send":
                return await self._api_send_message(http_client, account, params)
            elif endpoint == "/edit":
                return await self._api_edit_message(http_client, account, params)
            elif endpoint == "/recall":
                return await self._api_recall_message(http_client, account, params)
            elif endpoint == "/recall_batch":
                return await self._api_recall_batch(http_client, account, params)
            elif endpoint == "/list":
                return await self._api_list_messages(http_client, params)
            elif endpoint == "/list_by_seq":
                return await self._api_list_by_seq(http_client, params)
            elif endpoint == "/list_by_mid_seq":
                return await self._api_list_by_mid_seq(http_client, params)
            elif endpoint == "/list_edit_record":
                return await self._api_list_edit_record(http_client, params)
            elif endpoint == "/button_report":
                return await self._api_button_report(http_client, params)
            else:
                return {
                    "status": "failed",
                    "retcode": 10001,
                    "data": None,
                    "message_id": "",
                    "message": f"不支持的端点: {endpoint}",
                    "yunhu_user_raw": {}
                }

        except Exception as e:
            self.logger.error(f"call_api 调用失败: {e}")
            return {
                "status": "failed",
                "retcode": 34000,
                "data": None,
                "message_id": "",
                "message": str(e),
                "yunhu_user_raw": {}
            }

    async def _api_send_message(self, http_client: YunhuHTTPClient,
                              account: Dict[str, Any],
                              params: Dict[str, Any]) -> Dict[str, Any]:
        """发送消息 API"""
        import uuid

        target_type = params.get("target_type", "group")
        target_id = params.get("target_id")
        message = params.get("message")

        # 映射 target_type 到 chat_type
        chat_type = CHAT_TYPE_MAP.get(target_type, 2)

        # 构建消息内容
        content = {"text": message.get("text", "")}
        if "buttons" in message:
            import json
            content["buttons"] = json.dumps(message["buttons"])
        if "mentioned_id" in message:
            content["mentioned_id"] = message["mentioned_id"]
        if "quote_msg_id" in message:
            content["quote_msg_text"] = ""

        # 发送消息
        response = await http_client.send_message(
            chat_id=target_id,
            chat_type=chat_type,
            msg_id=uuid.uuid4().hex,
            msg_type=message.get("msg_type", 1),
            content=content,
            quote_msg_id=message.get("quote_msg_id"),
            media=message.get("media")
        )

        # 格式化响应
        return ResponseFormatter.from_platform_response(response)

    async def _api_edit_message(self, http_client: YunhuHTTPClient,
                             account: Dict[str, Any],
                             params: Dict[str, Any]) -> Dict[str, Any]:
        """编辑消息 API"""
        target_type = params.get("target_type", "group")
        target_id = params.get("target_id")
        msg_id = params.get("msg_id")
        text = params.get("text", "")
        content_type = params.get("content_type", "text")

        # 映射 content_type
        from .utils.constants import CONTENT_TYPE_MAP
        ct = CONTENT_TYPE_MAP.get(content_type, 1)

        # 映射 target_type 到 chat_type
        chat_type = CHAT_TYPE_MAP.get(target_type, 2)

        response = await http_client.edit_message(
            chat_id=target_id,
            chat_type=chat_type,
            msg_id=msg_id,
            text=text,
            msg_type=ct
        )

        return ResponseFormatter.from_platform_response(response)

    async def _api_recall_message(self, http_client: YunhuHTTPClient,
                               account: Dict[str, Any],
                               params: Dict[str, Any]) -> Dict[str, Any]:
        """撤回消息 API"""
        target_type = params.get("target_type", "group")
        target_id = params.get("target_id")
        msg_id = params.get("msg_id")

        # 映射 target_type 到 chat_type
        chat_type = CHAT_TYPE_MAP.get(target_type, 2)

        response = await http_client.recall_message(
            chat_id=target_id,
            chat_type=chat_type,
            msg_id=msg_id
        )

        return ResponseFormatter.from_platform_response(response)

    async def _api_recall_batch(self, http_client: YunhuHTTPClient,
                              account: Dict[str, Any],
                              params: Dict[str, Any]) -> Dict[str, Any]:
        """批量撤回消息 API"""
        target_type = params.get("target_type", "group")
        target_id = params.get("target_id")
        msg_id_list = params.get("msg_id_list", [])

        # 映射 target_type 到 chat_type
        chat_type = CHAT_TYPE_MAP.get(target_type, 2)

        response = await http_client.recall_msg_batch(
            chat_id=target_id,
            chat_type=chat_type,
            msg_id_list=msg_id_list
        )

        return ResponseFormatter.from_platform_response(response)

    async def _api_list_messages(self, http_client: YunhuHTTPClient,
                               params: Dict[str, Any]) -> Dict[str, Any]:
        """获取消息列表 API"""
        chat_id = params.get("chat_id")
        chat_type = params.get("chat_type", 2)
        msg_count = params.get("msg_count", 1)
        msg_id = params.get("msg_id", "")

        response = await http_client.list_msg(
            chat_id=chat_id,
            chat_type=chat_type,
            msg_count=msg_count,
            msg_id=msg_id
        )

        return ResponseFormatter.from_platform_response(response)

    async def _api_list_by_seq(self, http_client: YunhuHTTPClient,
                              params: Dict[str, Any]) -> Dict[str, Any]:
        """通过序列获取消息 API"""
        chat_id = params.get("chat_id")
        chat_type = params.get("chat_type", 2)
        msg_start = params.get("msg_start", 0)

        response = await http_client.list_msg_by_seq(
            chat_id=chat_id,
            chat_type=chat_type,
            msg_start=msg_start
        )

        return ResponseFormatter.from_platform_response(response)

    async def _api_list_by_mid_seq(self, http_client: YunhuHTTPClient,
                                   params: Dict[str, Any]) -> Dict[str, Any]:
        """通过消息ID和序列获取消息 API"""
        chat_id = params.get("chat_id")
        chat_type = params.get("chat_type", 2)
        msg_id = params.get("msg_id", "")
        msg_count = params.get("msg_count", 1)
        msg_seq = params.get("msg_seq", -1)

        response = await http_client.list_msg_by_mid_seq(
            chat_id=chat_id,
            chat_type=chat_type,
            msg_id=msg_id,
            msg_count=msg_count,
            msg_seq=msg_seq
        )

        return ResponseFormatter.from_platform_response(response)

    async def _api_list_edit_record(self, http_client: YunhuHTTPClient,
                                   params: Dict[str, Any]) -> Dict[str, Any]:
        """获取消息编辑记录 API"""
        msg_id = params.get("msg_id")
        size = params.get("size", 10)
        page = params.get("page", 1)

        response = await http_client.list_msg_edit_record(
            msg_id=msg_id,
            size=size,
            page=page
        )

        return ResponseFormatter.from_platform_response(response)

    async def _api_button_report(self, http_client: YunhuHTTPClient,
                                params: Dict[str, Any]) -> Dict[str, Any]:
        """按钮事件报告 API"""
        chat_id = params.get("chat_id")
        chat_type = params.get("chat_type", 2)
        msg_id = params.get("msg_id")
        user_id = params.get("user_id")
        button_value = params.get("button_value")

        response = await http_client.button_report(
            chat_id=chat_id,
            chat_type=chat_type,
            msg_id=msg_id,
            user_id=user_id,
            button_value=button_value
        )

        return ResponseFormatter.from_platform_response(response)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: user.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'user.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nuser.proto\x12\nyunhu_user\"Z\n\x13\x65mail_login_request\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\x12\x10\n\x08\x64\x65viceId\x18\x03 \x01(\t\x12\x10\n\x08platform\x18\x04 \x01(\t\"\xc0\x01\n\x14\x65mail_login_response\x12\x37\n\x06status\x18\x01 \x01(\x0b\x32\'.yunhu_user.email_login_response.Status\x12\x33\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32%.yunhu_user.email_login_response.Data\x1a#\n\x06Status\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0b\n\x03msg\x18\x02 \x01(\t\x1a\x15\n\x04\x44\x61ta\x12\r\n\x05token\x18\x01 \x01(\t\"\xcc\x01\n\x12user_info_response\x12\x35\n\x06status\x18\x01 \x01(\x0b\x32%.yunhu_user.user_info_response.Status\x12\x31\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32#.yunhu_user.user_info_response.Data\x1a\x16\n\x06Status\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x1a\x34\n\x04\x44\x61ta\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\navatar_url\x18\x04 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'user_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EMAIL_LOGIN_REQUEST']._serialized_start=26
  _globals['_EMAIL_LOGIN_REQUEST']._serialized_end=116
  _globals['_EMAIL_LOGIN_RESPONSE']._serialized_start=119
  _globals['_EMAIL_LOGIN_RESPONSE']._serialized_end=311
  _globals['_EMAIL_LOGIN_RESPONSE_STATUS']._serialized_start=253
  _globals['_EMAIL_LOGIN_RESPONSE_STATUS']._serialized_end=288
  _globals['_EMAIL_LOGIN_RESPONSE_DATA']._serialized_start=290
  _globals['_EMAIL_LOGIN_RESPONSE_DATA']._serialized_end=311
  _globals['_USER_INFO_RESPONSE']._serialized_start=314
  _globals['_USER_INFO_RESPONSE']._serialized_end=518
  _globals['_USER_INFO_RESPONSE_STATUS']._serialized_start=253
  _globals['_USER_INFO_RESPONSE_STATUS']._serialized_end=275
  _globals['_USER_INFO_RESPONSE_DATA']._serialized_start=466
  _globals['_USER_INFO_RESPONSE_DATA']._serialized_end=518
# @@protoc_insertion_point(module_scope)



import json
import uuid
import websockets
from typing import Optional, Dict, Any, AsyncIterator
from google.protobuf import json_format
from ..proto import ws_pb2


class YunhuWSClient:
    WS_URL = "wss://chat-ws-go.jwzhd.com/ws"
    
    def __init__(self, token: str, user_id: str, 
                 platform: str = "windows", device_id: Optional[str] = None,
                 timeout: int = 70, decode: bool = True, 
                 mode: str = "black", blacklist: list = None):
        """
        初始化 WebSocket 客户端
        
        :param token: 用户 token
        :param user_id: 用户 ID
        :param platform: 平台
        :param device_id: 设备 ID
        :param timeout: 超时时间（秒）
        :param decode: 是否解码消息
        :param mode: 过滤模式 (black/white)
        :param blacklist: 黑名单或白名单列表
        """
        self.token = token
        self.user_id = user_id
        self.platform = platform
        self.device_id = device_id or uuid.uuid4().hex
        # 确保 timeout 为整数类型
        self.timeout = int(timeout) if timeout is not None else 70
        self.decode = decode
        self.mode = mode
        self.filter_list = blacklist or []
        
        self.ws = None
        self._running = False
    
    async def connect(self) -> AsyncIterator[Dict[str, Any]]:
        """
        连接到 WebSocket 并监听事件
        
        :return: 异步迭代器，返回解析后的事件
        """
        self._running = True
        login_data = {
            "seq": uuid.uuid4().hex,
            "cmd": "login",
            "data": {
                "userId": self.user_id,
                "token": self.token,
                "platform": self.platform,
                "deviceId": self.device_id
            }
        }
        
        try:
            # 此处兼容：使用 ping_interval 和 ping_timeout 替代 timeout
            # 非必要勿修改
            async with websockets.connect(
                self.WS_URL,
                ping_interval=30,
                ping_timeout=self.timeout,
                close_timeout=10
            ) as ws:
                self.ws = ws
                
                # 发送登录消息
                login_json = json.dumps(login_data)
                await ws.send(login_json)
                
                # 启动心跳任务
                import asyncio
                heartbeat_task = asyncio.create_task(self._heartbeat_loop(ws))
                
                try:
                    # 接收消息循环
                    while self._running:
                        try:
                            # 等待接收消息
                            recv_task = asyncio.create_task(ws.recv())
                            message = await asyncio.wait_for(recv_task, timeout=self.timeout)
                            
                            if not self._running:
                                break
                            
                            # 解析事件
                            event = self._parse_message(message)
                            
                            # 检查是否需要过滤
                            if self._should_filter(event):
                                continue
                            
                            # 是否解码
                            if self.decode and event:
                                yield event
                            elif not self.decode:
                                yield message
                                
                        except asyncio.TimeoutError:
                            # 超时，继续等待
                            continue
                        except Exception as e:
                            # 其他错误，打印日志
                            if self._running:
                                print(f"WebSocket 消息处理错误: {e}")
                            break
                finally:
                    # 取消心跳任务
                    heartbeat_task.cancel()
                    try:
                        await heartbeat_task
                    except asyncio.CancelledError:
                        pass
                    
        except Exception as e:
            print(f"WebSocket 连接错误: {e}")
        finally:
            self.ws = None
            self._running = False
    
    async def _heartbeat_loop(self, ws):
        import asyncio
        heartbeat_data = {
            "seq": uuid.uuid4().hex,
            "cmd": "heartbeat",
            "data": {}
        }
        
        while self._running:
            try:
                heartbeat_json = json.dumps(heartbeat_data)
                await ws.send(heartbeat_json)
                await asyncio.sleep(30)  # 30 秒发送一次心跳
            except asyncio.CancelledError:
                break
            except Exception as e:
                if self._running:
                    print(f"心跳发送错误: {e}")
                break
    
    def _parse_message(self, message: bytes) -> Optional[Dict[str, Any]]:
        """
        解析 WebSocket 消息

        :param message: 消息二进制数据（Protobuf 格式）
        :return: 解析后的事件字典
        """
        # 命令类型映射
        cmd_map = {
            "push_message": ws_pb2.push_message,
            "edit_message": ws_pb2.edit_message,
            "file_send_message": ws_pb2.file_send_message,
            "bot_board_message": ws_pb2.bot_board_message,
            "stream_message": ws_pb2.stream_message,
            "draft_input": ws_pb2.draft_input,
        }

        try:
            # 先解析为 heartbeat_ack 获取命令类型
            msg_temp = ws_pb2.heartbeat_ack()
            msg_temp.ParseFromString(message)

            # 检查是否需要过滤
            if self._should_filter_by_cmd(msg_temp.info.cmd):
                return None

            # 根据命令类型解析
            if msg_temp.info.cmd == "heartbeat_ack":
                # 心跳响应
                return json_format.MessageToDict(msg_temp, preserving_proto_field_name=True)
            elif msg_temp.info.cmd in cmd_map:
                # 具体消息类型
                msg_class = cmd_map[msg_temp.info.cmd]
                msg_data = msg_class()
                msg_data.ParseFromString(message)
                return json_format.MessageToDict(msg_data, preserving_proto_field_name=True)
            else:
                return None

        except Exception as e:
            print(f"Protobuf 解析错误: {e}")
            return None
    
    def _should_filter_by_cmd(self, cmd: str) -> bool:
        """
        根据命令检查是否需要过滤
        
        :param cmd: 命令字符串
        :return: True 表示需要过滤
        """
        if self.mode == "black":
            # 黑名单模式：在列表中则过滤
            return cmd in self.filter_list
        elif self.mode == "white":
            # 白名单模式：不在列表中则过滤
            return cmd not in self.filter_list
        return False
    
    def _should_filter(self, event: Optional[Dict[str, Any]]) -> bool:
        """
        检查事件是否需要被过滤
        
        :param event: 事件数据
        :return: True 表示需要过滤
        """
        if not event:
            return True
        
        cmd = event.get("info", {}).get("cmd", "")
        
        if self.mode == "black":
            # 黑名单模式：在列表中则过滤
            return cmd in self.filter_list
        elif self.mode == "white":
            # 白名单模式：不在列表中则过滤
            return cmd not in self.filter_list
        
        return False
    
    async def close(self):
        self._running = False
        if self.ws:
            try:
                await self.ws.close()
            except Exception:
                pass
            self.ws = None


import asyncio
import uuid
import json
from typing import Union, Optional, List, Dict, Any

from .utils.constants import (
    MessageTypes,
    ChatType,
    CHAT_TYPE_MAP,
    CONTENT_TYPE_MAP
)
from .utils.response_formatter import ResponseFormatter
from .utils.media_handler import MediaHandler
from ErisPulse.Core.Bases import BaseAdapter

class Send(BaseAdapter.Send):
    
    def __init__(
        self, 
        adapter_obj,
        target_type: Optional[str] = None,
        target_id: Optional[str] = None,
        account_id: Optional[str] = None
    ):
        self._adapter = adapter_obj
        self._target_type = target_type
        self._target_id = target_id
        self._account_id = account_id
        
        # 链式修饰属性
        self._at_user_ids = []
        self._at_all = False
        self._reply_message_id = None
        self._buttons = None
        
        # 初始化媒体处理器
        self._media_handler = None
    
    def At(self, user_id: str) -> 'Send':
        if user_id not in self._at_user_ids:
            self._at_user_ids.append(user_id)
        return self
    
    def AtAll(self) -> 'Send':
        self._at_all = True
        return self
    
    def Reply(self, message_id: str) -> 'Send':
        self._reply_message_id = message_id
        return self
    
    def Buttons(self, buttons: List[Dict[str, Any]]) -> 'Send':
        self._buttons = buttons
        return self
    
    def _get_account(self) -> Optional[Dict[str, Any]]:
        """
        获取当前账户
        """
        if self._account_id:
            return self._adapter._get_account_by_id(self._account_id)
        
        if self._adapter.accounts:
            return next(iter(self._adapter.accounts.values()))
        
        return None
    
    def _get_http_client(self):
        """获取当前账户的 HTTP 客户端"""
        from .client.http import YunhuHTTPClient
        account = self._get_account()
        if account:
            account_name = account["name"]
            return self._adapter._get_http_client(account_name)
        return None
    
    def _get_media_handler(self) -> MediaHandler:
        """
        获取或创建媒体处理器
        """
        if self._media_handler is None:
            http_client = self._get_http_client()
            self._media_handler = MediaHandler(
                http_client,
                self._adapter.logger
            )
        return self._media_handler
    
    def _map_detail_type_to_chat_type(self) -> int:
        """
        将 OneBot12 detail_type 映射到云湖 chat_type
        
        :return: 云湖 chat_type
        """
        return CHAT_TYPE_MAP.get(self._target_type, ChatType.GROUP)
    
    def _build_send_content(self, quote_text: str = "") -> Dict[str, Any]:
        """
        构建发送消息内容
        
        :param quote_text: 引用消息的文本（如果需要回复）
        :return: 消息内容字典
        """
        content = {}
        
        # 添加 @用户
        if self._at_user_ids:
            content["mentioned_id"] = self._at_user_ids
        
        # 添加回复
        if self._reply_message_id:
            content["quote_msg_id"] = self._reply_message_id
            # 使用传入的引用文本
            if quote_text:
                content["quote_msg_text"] = quote_text
        
        # 添加按钮
        if self._buttons:
            content["buttons"] = json.dumps(self._buttons)
        
        return content
    
    async def _get_quote_text_if_needed(self) -> str:
        """
        如果需要回复消息，获取被引用消息的真实文本
        
        :return: 引用消息文本，如果不需要回复则返回空字符串
        """
        if self._reply_message_id:
            return await self._get_quote_message_text(self._reply_message_id)
        return ""
    
    async def _get_quote_message_text(self, message_id: str) -> str:
        """
        获取被引用消息的文本内容
        
        :param message_id: 消息 ID
        :return: 消息文本内容
        """
        try:
            http_client = self._get_http_client()
            if not http_client:
                return ""
            
            response = await http_client.list_msg_by_mid_seq(
                chat_id=self._target_id,
                chat_type=self._map_detail_type_to_chat_type(),
                msg_id=message_id,
                msg_count=1
            )
            
            # 解析响应获取消息文本
            if response and response.get("status", {}).get("code") == 1:
                messages = response.get("msg", [])
                if messages and len(messages) > 0:
                    msg = messages[0]
                    content = msg.get("content", {})
                    text = content.get("text", "")
                    return text
            
            return ""
        except Exception as e:
            self._adapter.logger.error(f"获取引用消息文本失败: {e}")
            return ""
    
    async def _send_text_like(
        self,
        text: str,
        msg_type: int,
        buttons: Optional[List] = None
    ) -> Dict[str, Any]:
        """
        统一发送文本类消息
        
        :param text: 文本内容
        :param msg_type: 消息类型
        :param buttons: 按钮列表
        :return: 发送结果
        """
        http_client = self._get_http_client()
        if not http_client:
            return ResponseFormatter.failed(10003, "没有可用的账户")
        
        # 获取引用消息的真实文本
        quote_text = await self._get_quote_text_if_needed()
        
        # 构建发送数据
        send_content = self._build_send_content(quote_text)
        send_content["text"] = text
        if buttons:
            send_content["buttons"] = json.dumps(buttons)
        
        try:
            msg_id = uuid.uuid4().hex
            response = await http_client.send_message(
                chat_id=self._target_id,
                chat_type=self._map_detail_type_to_chat_type(),
                msg_id=msg_id,
                msg_type=msg_type,
                content=send_content,
                quote_msg_id=self._reply_message_id,
                media=None
            )
            return ResponseFormatter.from_platform_response(response, msg_id)
        except Exception as e:
            self._adapter.logger.error(f"发送文本类消息失败: {e}")
            return ResponseFormatter.error(e)
    
    async def _send_media(
        self,
        file: Union[str, bytes],
        file_type: str,
        msg_type: int,
        extra_content: Optional[Dict] = None,
        buttons: Optional[List] = None
    ) -> Dict[str, Any]:
        """
        统一发送媒体消息
        
        :param file: 文件 URL 或二进制数据
        :param file_type: 文件类型
        :param msg_type: 消息类型
        :param extra_content: 额外内容（如 file_size, audio_time 等）
        :param buttons: 按钮列表
        :return: 发送结果
        """
        try:
            # 处理文件
            media_handler = self._get_media_handler()
            process_result = await media_handler.process_file(file, file_type)
            
            if not process_result:
                return ResponseFormatter.failed(10003, "文件处理失败")
            
            # 获取引用消息的真实文本
            quote_text = await self._get_quote_text_if_needed()
            
            # 构建发送内容
            send_content = self._build_send_content(quote_text)
            
            # 根据文件类型设置内容字段
            content_field_map = {
                "image": "image",
                "video": "video",
                "audio": "audio",
                "file": "file"
            }
            content_field = content_field_map.get(file_type, "file")
            send_content[content_field] = process_result["key"]
            
            # 自动添加文件大小
            send_content["file_size"] = process_result["file_size"]
            
            # 自动添加文件名（仅文件类型）
            if file_type == "file" and process_result.get("filename"):
                send_content["file_name"] = process_result["filename"]
            
            # 添加额外内容（允许覆盖自动检测的值）
            if extra_content:
                send_content.update(extra_content)
            
            # 添加按钮
            if buttons:
                send_content["buttons"] = json.dumps(buttons)
            
            # 发送消息
            http_client = self._get_http_client()
            msg_id = uuid.uuid4().hex
            response = await http_client.send_message(
                chat_id=self._target_id,
                chat_type=self._map_detail_type_to_chat_type(),
                msg_id=msg_id,
                msg_type=msg_type,
                content=send_content,
                quote_msg_id=self._reply_message_id,
                media=None
            )
            
            return ResponseFormatter.from_platform_response(response, msg_id)
        
        except Exception as e:
            self._adapter.logger.error(f"发送媒体消息失败: {e}")
            return ResponseFormatter.error(e)
    
    # ============ 公开发送方法 ============
    
    def Text(self, text: str, buttons: Optional[List] = None) -> asyncio.Task:
        """
        发送文本消息
        
        :param text: 文本内容
        :param buttons: 按钮列表
        :return: asyncio.Task
        """
        return asyncio.create_task(
            self._send_text_like(text, MessageTypes.TEXT, buttons)
        )
    
    def Html(self, html: str, buttons: Optional[List] = None) -> asyncio.Task:
        """
        发送 HTML 消息
        
        :param html: HTML 内容
        :param buttons: 按钮列表
        :return: asyncio.Task
        """
        return asyncio.create_task(
            self._send_text_like(html, MessageTypes.HTML, buttons)
        )
    
    def Markdown(self, markdown: str, buttons: Optional[List] = None) -> asyncio.Task:
        """
        发送 Markdown 消息
        
        :param markdown: Markdown 内容
        :param buttons: 按钮列表
        :return: asyncio.Task
        """
        return asyncio.create_task(
            self._send_text_like(markdown, MessageTypes.MARKDOWN, buttons)
        )
    
    def Image(self, file: Union[str, bytes], buttons: Optional[List] = None) -> asyncio.Task:
        """
        发送图片消息
        
        :param file: 图片 URL 或二进制数据
        :param buttons: 按钮列表
        :return: asyncio.Task
        """
        return asyncio.create_task(
            self._send_media(file, "image", MessageTypes.IMAGE, buttons=buttons)
        )
    
    def Video(self, file: Union[str, bytes], buttons: Optional[List] = None) -> asyncio.Task:
        """
        发送视频消息
        
        :param file: 视频 URL 或二进制数据
        :param buttons: 按钮列表
        :return: asyncio.Task
        """
        return asyncio.create_task(
            self._send_media(file, "video", MessageTypes.VIDEO, buttons=buttons)
        )
    
    def Audio(self, file: Union[str, bytes], buttons: Optional[List] = None) -> asyncio.Task:
        """
        发送语音消息
        
        :param file: 语音 URL 或二进制数据
        :param buttons: 按钮列表
        :return: asyncio.Task
        """
        async def _send_with_duration():
            try:
                # 处理文件
                media_handler = self._get_media_handler()
                process_result = await media_handler.process_file(file, "audio")
                
                if not process_result:
                    return ResponseFormatter.failed(10003, "文件处理失败")
                
                # 获取音频时长
                file_data = None
                if isinstance(file, bytes):
                    file_data = file
                else:
                    file_data = await media_handler._download_file(file)
                
                if file_data:
                    audio_time = await media_handler.get_audio_duration(file_data)
                else:
                    audio_time = 0
                
                # 获取引用消息的真实文本
                quote_text = await self._get_quote_text_if_needed()
                
                # 构建发送内容
                send_content = self._build_send_content(quote_text)
                send_content["audio"] = process_result["key"]
                send_content["audio_time"] = audio_time
                send_content["file_size"] = process_result["file_size"]
                
                if buttons:
                    send_content["buttons"] = json.dumps(buttons)
                
                # 发送消息
                http_client = self._get_http_client()
                msg_id = uuid.uuid4().hex
                response = await http_client.send_message(
                    chat_id=self._target_id,
                    chat_type=self._map_detail_type_to_chat_type(),
                    msg_id=msg_id,
                    msg_type=MessageTypes.AUDIO,
                    content=send_content,
                    quote_msg_id=self._reply_message_id,
                    media=None
                )
                
                return ResponseFormatter.from_platform_response(response, msg_id)
            
            except Exception as e:
                self._adapter.logger.error(f"发送语音消息失败: {e}")
                return ResponseFormatter.error(e)
        
        return asyncio.create_task(_send_with_duration())
    
    def Voice(self, file: Union[str, bytes], buttons: Optional[List] = None) -> asyncio.Task:
        """
        发送语音消息（Audio 的别名）
        
        :param file: 语音 URL 或二进制数据
        :param buttons: 按钮列表
        :return: asyncio.Task
        """
        return self.Audio(file, buttons)
    
    def File(self, file: Union[str, bytes], file_name: Optional[str] = None, buttons: Optional[List] = None) -> asyncio.Task:
        """
        发送文件消息
        
        :param file: 文件 URL 或二进制数据
        :param file_name: 文件名（可选，不提供时自动检测）
        :param buttons: 按钮列表
        :return: asyncio.Task
        """
        extra_content = {}
        if file_name:
            extra_content["file_name"] = file_name
        
        return asyncio.create_task(
            self._send_media(file, "file", MessageTypes.FILE, extra_content, buttons)
        )
    
    def Face(self, file: Union[str, bytes], buttons: Optional[List] = None) -> asyncio.Task:
        """
        发送表情/贴纸消息
        
        :param file: 表情 URL、二进制数据或表情ID
        :param buttons: 按钮列表
        :return: asyncio.Task
        """
        async def _send_face():
            try:
                http_client = self._get_http_client()
                if not http_client:
                    return ResponseFormatter.failed(10003, "没有可用的账户")
                
                # 获取引用消息的真实文本
                quote_text = await self._get_quote_text_if_needed()
                
                send_content = self._build_send_content(quote_text)
                
                # URL 方式 - 直接发送 URL
                if isinstance(file, str) and file.startswith(("http://", "https://")):
                    send_content["sticker_url"] = file
                # 表情ID 方式
                elif isinstance(file, str):
                    send_content["expression_id"] = file
                # 二进制方式 - 上传后发送
                elif isinstance(file, bytes):
                    media_handler = self._get_media_handler()
                    process_result = await media_handler.process_file(file, "image")
                    if not process_result:
                        return ResponseFormatter.failed(10003, "文件处理失败")
                    send_content["sticker_url"] = process_result["key"]
                else:
                    return ResponseFormatter.failed(10003, "不支持的文件格式")
                
                if buttons:
                    send_content["buttons"] = json.dumps(buttons)
                
                msg_id = uuid.uuid4().hex
                response = await http_client.send_message(
                    chat_id=self._target_id,
                    chat_type=self._map_detail_type_to_chat_type(),
                    msg_id=msg_id,
                    msg_type=MessageTypes.STICKER,
                    content=send_content,
                    quote_msg_id=self._reply_message_id,
                    media=None
                )
                
                return ResponseFormatter.from_platform_response(response, msg_id)
            
            except Exception as e:
                self._adapter.logger.error(f"发送表情消息失败: {e}")
                return ResponseFormatter.error(e)
        
        return asyncio.create_task(_send_face())
    
    def Edit(self, msg_id: str, text: str, content_type: str = "text") -> asyncio.Task:
        """
        编辑消息
        
        :param msg_id: 消息 ID
        :param text: 新消息内容
        :param content_type: 内容类型
        :return: asyncio.Task
        """
        async def _edit():
            http_client = self._get_http_client()
            if not http_client:
                return ResponseFormatter.failed(10003, "没有可用的账户")
            
            try:
                msg_type = CONTENT_TYPE_MAP.get(content_type, MessageTypes.TEXT)
                
                response = await http_client.edit_message(
                    chat_id=self._target_id,
                    chat_type=self._map_detail_type_to_chat_type(),
                    msg_id=msg_id,
                    text=text,
                    msg_type=msg_type,
                    quote_msg_id=self._reply_message_id if self._reply_message_id else None,
                    buttons=json.dumps(self._buttons) if self._buttons else None
                )
                
                return ResponseFormatter.from_platform_response(response)
            
            except Exception as e:
                self._adapter.logger.error(f"编辑消息失败: {e}")
                return ResponseFormatter.error(e)
        
        return asyncio.create_task(_edit())
    
    def Recall(self, msg_id: str) -> asyncio.Task:
        """
        撤回消息
        
        :param msg_id: 消息 ID
        :return: asyncio.Task
        """
        async def _recall():
            http_client = self._get_http_client()
            if not http_client:
                return ResponseFormatter.failed(10003, "没有可用的账户")
            
            try:
                response = await http_client.recall_message(
                    chat_id=self._target_id,
                    chat_type=self._map_detail_type_to_chat_type(),
                    msg_id=msg_id
                )
                return ResponseFormatter.from_platform_response(response)
            
            except Exception as e:
                self._adapter.logger.error(f"撤回消息失败: {e}")
                return ResponseFormatter.error(e)
        
        return asyncio.create_task(_recall())
    
    def Raw_ob12(self, message: Union[List, Dict]) -> asyncio.Task:
        """
        发送 OneBot12 格式消息
        
        :param message: OneBot12 消息段或消息段数组
        :return: asyncio.Task
        """
        # 处理单条消息段的情况
        if isinstance(message, dict):
            message = [message]
        
        # 分组消息段，云湖不支持组合消息
        grouped_messages = self._group_ob12_messages(message)
        
        # 定义异步发送函数
        async def _send_grouped_messages():
            results = []
            for msg_group in grouped_messages:
                result = await self._send_ob12_group(msg_group)
                results.append(result)
            
            # 返回最后一个结果作为主结果
            return results[-1] if results else None
        
        return asyncio.create_task(_send_grouped_messages())
    
    def _group_ob12_messages(self, message: List[Dict]) -> List[List[Dict]]:
        """
        将 OneBot12 消息段数组分组
        
        :param message: OneBot12 消息段数组
        :return: 分组后的消息段数组列表
        """
        groups = []
        current_group = []
        
        # 定义可以合并到文本组的消息类型
        text_mergeable_types = ["text", "mention"]
        
        for segment in message:
            seg_type = segment.get("type", "")
            
            # 回复消息可以附加到任何组
            if seg_type == "reply":
                if not current_group:
                    current_group.append(segment)
                else:
                    current_group.append(segment)
                continue
            
            # 文本消息、@可以合并
            if seg_type in text_mergeable_types:
                if not current_group or all(s.get("type") in text_mergeable_types or s.get("type") == "reply" for s in current_group):
                    current_group.append(segment)
                else:
                    if current_group:
                        groups.append(current_group)
                    current_group = [segment]
            
            # 其他消息类型单独成组
            else:
                if current_group:
                    groups.append(current_group)
                groups.append([segment])
                current_group = []
        
        # 保存最后一组
        if current_group:
            groups.append(current_group)
        
        return groups
    
    async def _send_ob12_group(self, msg_group: List[Dict]) -> Dict:
        """
        发送一组消息段
        
        :param msg_group: 一组消息段
        :return: 发送结果
        """
        if not msg_group:
            return None
        
        # 获取第一个段来确定类型
        first_segment = msg_group[0]
        seg_type = first_segment.get("type", "")
        
        # 检查是否有 reply 消息段
        reply_message_id = None
        for segment in msg_group:
            if segment.get("type") == "reply":
                reply_message_id = segment.get("data", {}).get("message_id")
                break
        
        # 处理文本组
        if seg_type in ["text", "mention"]:
            text_parts = []
            for segment in msg_group:
                s_type = segment.get("type", "")
                s_data = segment.get("data", {})
                if s_type == "text":
                    text_parts.append(s_data.get("text", ""))
                elif s_type == "mention":
                    user_id = s_data.get("user_id", "")
                    text_parts.append(f"@{user_id}")
            
            # 添加链式修饰的@用户
            if self._at_user_ids:
                at_text = " ".join([f"@{uid}" for uid in self._at_user_ids])
                text_parts.insert(0, at_text)
            
            text = " ".join(text_parts) or " "
            
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.Text(text, buttons=self._buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result
        
        # 处理其他类型的消息
        seg_data = first_segment.get("data", {})
        
        # 图片
        if seg_type == "image":
            file_url = seg_data.get("file") or seg_data.get("url", "")
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.Image(file_url, buttons=self._buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result
        
        # 音频
        elif seg_type == "audio":
            file_url = seg_data.get("file") or seg_data.get("url", "")
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.Audio(file_url, buttons=self._buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result
        
        # 视频
        elif seg_type == "video":
            file_url = seg_data.get("file") or seg_data.get("url", "")
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.Video(file_url, buttons=self._buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result
        
        # 文件
        elif seg_type == "file":
            file_url = seg_data.get("file") or seg_data.get("url", "")
            file_name = seg_data.get("file_name", "")
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.File(file_url, file_name=file_name, buttons=self._buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result
        
        # Markdown
        elif seg_type == "markdown":
            markdown_text = seg_data.get("markdown", "")
            seg_buttons = seg_data.get("buttons")
            final_buttons = self._buttons if self._buttons is not None else seg_buttons
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.Markdown(markdown_text, buttons=final_buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result
        
        # HTML
        elif seg_type == "html":
            html_text = seg_data.get("html", "")
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.Html(html_text, buttons=self._buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result
        
        # 回复（单独一个reply的情况）
        elif seg_type == "reply":
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.Text("", buttons=self._buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result
        
        # 云湖特有消息段
        elif seg_type.startswith("yunhu_"):
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.Text(str(seg_data), buttons=self._buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result
        
        # 其他未知类型
        else:
            old_reply_id = self._reply_message_id
            if reply_message_id:
                self._reply_message_id = reply_message_id
            task = self.Text(str(seg_data), buttons=self._buttons)
            self._reply_message_id = old_reply_id
            result = await task
            return result


from .constants import (
    MessageTypes,
    ChatType,
    CHAT_TYPE_MAP,
    CONTENT_TYPE_MAP
)
from .response_formatter import ResponseFormatter
from .media_handler import MediaHandler

__all__ = [
    "MessageTypes",
    "ChatType",
    "CHAT_TYPE_MAP",
    "CONTENT_TYPE_MAP",
    "ResponseFormatter",
    "MediaHandler"
]


import time
import uuid
from typing import Optional, Dict, Any, List
from .proto import ws_pb2 as ws_pb2


class YunhuUserConverter:
    
    def convert(self, ws_event: Dict[str, Any]) -> Optional[Dict[str, Any]]:
        """
        将云湖 WebSocket 事件转换为 OneBot12 标准格式

        :param ws_event: 云湖 WebSocket 原始事件
        :return: OneBot12 标准事件，如果不支持的事件类型则返回 None
        """
        if not isinstance(ws_event, dict):
            return None

        info = ws_event.get("info", {})
        cmd = info.get("cmd", "")

        # 根据命令类型分发处理
        if cmd == "push_message":
            return self._convert_message(ws_event)
        elif cmd == "edit_message":
            return self._convert_edit_notice(ws_event)
        elif cmd == "file_send_message":
            return self._convert_file_send_message(ws_event)
        elif cmd == "bot_board_message":
            return self._convert_bot_board_message(ws_event)
        # 暂时注释掉流式消息处理（stream_message），因为频繁发送造成困扰
        # elif cmd == "stream_message":
        #     return self._convert_stream_message(ws_event)
        # draft_input 暂不处理（非必要功能）
        else:
            # 不支持的事件类型
            return None
    
    def _convert_message(self, ws_event: Dict[str, Any]) -> Dict[str, Any]:
        """
        转换推送消息事件为 OneBot12 message 事件
        
        :param ws_event: push_message 事件
        :return: OneBot12 message 事件
        """
        data = ws_event.get("data", {})
        msg = data.get("msg", {})
        info = ws_event.get("info", {})
        
        # 提取基本信息
        chat_type = msg.get("chat_type", 1)  # 1=user, 2=group, 3=bot
        chat_id = msg.get("chat_id", "")
        content_type = msg.get("content_type", 1)
        content = msg.get("content", {})
        sender = msg.get("sender", {})
        
        # 时间戳转换（毫秒转秒）
        timestamp = int(int(msg.get("timestamp", time.time() * 1000)) / 1000)
        
        # 转换 detail_type
        detail_type = self._map_chat_type(chat_type)
        
        # 构建消息段
        message_segments = self._convert_content_to_segments(content, content_type, msg)
        alt_message = self._build_alt_message(message_segments)
        
        # 提取发送者信息
        user_id = sender.get("chat_id", "")
        user_nickname = sender.get("name", "")
        
        # 构建 OneBot12 事件
        ob12_event = {
            "id": self._generate_event_id(msg),
            "time": int(time.time()),
            "type": "message",
            "detail_type": detail_type,
            "platform": "yunhu_user",
            "self": {
                "platform": "yunhu_user",
                "user_id": ""  # 由适配器填充为当前登录用户ID
            },
            "message": message_segments,
            "alt_message": alt_message,
            "user_id": user_id,
            "user_nickname": user_nickname,
            "yunhu_user_raw": ws_event,
            "yunhu_user_raw_type": "push_message"
        }
        
        # 群聊消息添加 group_id
        if detail_type == "group":
            ob12_event["group_id"] = chat_id
        elif detail_type == "bot":
            ob12_event["bot_id"] = chat_id
        
        # 消息ID
        message_id = msg.get("msg_id", "")
        if message_id:
            ob12_event["message_id"] = message_id
        
        return ob12_event
    
    def _convert_edit_notice(self, ws_event: Dict[str, Any]) -> Dict[str, Any]:
        """
        转换编辑消息事件为 OneBot12 notice 事件
        
        :param ws_event: edit_message 事件
        :return: OneBot12 notice 事件
        """
        data = ws_event.get("data", {})
        msg = data.get("msg", {})
        info = ws_event.get("info", {})
        
        # 提取基本信息
        message_id = msg.get("msg_id", "")
        chat_type = msg.get("chat_type", 1)
        chat_id = msg.get("chat_id", "")
        sender = msg.get("sender", {})
        
        # 提取编辑者信息
        user_id = sender.get("chat_id", "")
        user_nickname = sender.get("name", "")
        
        # 编辑时间
        edit_time = int(int(msg.get("edit_time", time.time() * 1000)) / 1000)
        
        # 转换 detail_type
        detail_type = self._map_chat_type(chat_type)
        
        # 构建 OneBot12 事件
        ob12_event = {
            "id": self._generate_event_id(ws_event),
            "time": int(time.time()),
            "type": "notice",
            "detail_type": "message_edit",
            "platform": "yunhu_user",
            "self": {
                "platform": "yunhu_user",
                "user_id": ""  # 由适配器填充
            },
            "message_id": message_id,
            "user_id": user_id,
            "user_nickname": user_nickname,
            "edit_time": edit_time,
            "yunhu_user_raw": ws_event,
            "yunhu_user_raw_type": "edit_message"
        }
        
        # 添加位置信息
        if detail_type == "group":
            ob12_event["group_id"] = chat_id
        elif detail_type == "bot":
            ob12_event["bot_id"] = chat_id
        
        return ob12_event
    
    def _convert_recall_notice(self, ws_event: Dict[str, Any]) -> Dict[str, Any]:
        """
        转换撤回消息事件为 OneBot12 notice 事件
        
        :param ws_event: recall_msg 事件
        :return: OneBot12 notice 事件
        """
        data = ws_event.get("data", {})
        msg = data.get("msg", {})
        info = ws_event.get("info", {})
        
        # 提取基本信息
        message_id = msg.get("msg_id", "")
        chat_type = msg.get("chat_type", 1)
        chat_id = msg.get("chat_id", "")
        sender = msg.get("sender", {})
        
        # 提取撤回者信息
        user_id = sender.get("chat_id", "")
        user_nickname = sender.get("name", "")
        
        # 撤回时间
        delete_time = int(int(msg.get("delete_time", time.time() * 1000)) / 1000)
        
        # 转换 detail_type
        detail_type = self._map_chat_type(chat_type)
        
        # 构建 OneBot12 事件
        ob12_event = {
            "id": self._generate_event_id(ws_event),
            "time": int(time.time()),
            "type": "notice",
            "detail_type": "message_delete",
            "platform": "yunhu_user",
            "self": {
                "platform": "yunhu_user",
                "user_id": ""  # 由适配器填充
            },
            "message_id": message_id,
            "user_id": user_id,
            "user_nickname": user_nickname,
            "delete_time": delete_time,
            "yunhu_user_raw": ws_event,
            "yunhu_user_raw_type": "recall_msg"
        }
        
        # 添加位置信息
        if detail_type == "group":
            ob12_event["group_id"] = chat_id
        elif detail_type == "bot":
            ob12_event["bot_id"] = chat_id
        
        return ob12_event
    
    def _convert_content_to_segments(
        self, 
        content: Dict[str, Any], 
        content_type: int,
        msg: Dict[str, Any]
    ) -> List[Dict[str, Any]]:
        """
        将云湖消息内容转换为 OneBot12 消息段
        
        :param content: 云湖消息内容
        :param content_type: 内容类型
        :param msg: 完整消息对象
        :return: OneBot12 消息段列表
        """
        segments = []
        
        # 文本消息
        if content_type == 1:
            text = content.get("text", "")
            if text:
                segments.append({"type": "text", "data": {"text": text}})
        
        # 图片消息
        elif content_type == 2:
            image_url = content.get("image_url", "")
            if image_url:
                segments.append({"type": "image", "data": {"file_id": image_url}})
        
        # Markdown 消息
        elif content_type == 3:
            markdown = content.get("text", "")
            if markdown:
                segments.append({"type": "text", "data": {"text": markdown}})
        
        # 文件消息
        elif content_type == 4:
            file_url = content.get("file_url", "")
            file_name = content.get("file_name", "")
            file_size = content.get("file_size", 0)
            if file_url:
                segments.append({
                    "type": "file",
                    "data": {
                        "file_id": file_url,
                        "file_name": file_name,
                        "file_size": file_size
                    }
                })
        
        # 表单消息
        elif content_type == 5:
            form = content.get("form", "")
            if form:
                segments.append({
                    "type": "yunhu_user_form",
                    "data": {"form": form}
                })
        
        # 文章消息
        elif content_type == 6:
            post_id = content.get("post_id", "")
            post_title = content.get("post_title", "")
            post_content = content.get("post_content", "")
            if post_id:
                segments.append({
                    "type": "yunhu_user_post",
                    "data": {
                        "post_id": post_id,
                        "post_title": post_title,
                        "post_content": post_content
                    }
                })
        
        # 表情/贴纸消息
        elif content_type == 7:
            sticker_url = content.get("sticker_url", "")
            if sticker_url:
                segments.append({
                    "type": "yunhu_user_sticker",
                    "data": {"file_id": sticker_url}
                })
        
        # HTML 消息
        elif content_type == 8:
            html = content.get("text", "")
            if html:
                segments.append({"type": "text", "data": {"text": html}})
        
        # 语音消息
        elif content_type == 11:
            audio_url = content.get("audio_url", "")
            audio_time = content.get("audio_time", 0)
            if audio_url:
                segments.append({
                    "type": "audio",
                    "data": {
                        "file_id": audio_url,
                        "duration": audio_time
                    }
                })
        
        # 视频消息
        # 假设视频消息使用 content_type 13 或其他
        video_url = content.get("video_url", "")
        if video_url:
            segments.append({
                "type": "video",
                "data": {"file_id": video_url}
            })
        
        # 处理按钮
        buttons_str = content.get("buttons", "")
        if buttons_str:
            import json
            try:
                buttons = json.loads(buttons_str) if isinstance(buttons_str, str) else buttons_str
                if buttons:
                    segments.append({
                        "type": "yunhu_user_button",
                        "data": {"buttons": buttons}
                    })
            except (json.JSONDecodeError, TypeError):
                pass
        
        # 处理 @用户
        mentioned_id = content.get("mentioned_id", [])
        if isinstance(mentioned_id, list) and mentioned_id:
            for uid in mentioned_id:
                segments.append({
                    "type": "mention",
                    "data": {"user_id": uid}
                })
        
        # 处理回复
        quote_msg_id = msg.get("quote_msg_id", "")
        if quote_msg_id:
            segments.append({
                "type": "reply",
                "data": {"message_id": quote_msg_id}
            })
        
        return segments if segments else [{"type": "text", "data": {"text": ""}}]
    
    def _build_alt_message(self, segments: List[Dict[str, Any]]) -> str:
        """
        构建消息段的备用文本
        
        :param segments: 消息段列表
        :return: 备用文本
        """
        texts = []
        for segment in segments:
            seg_type = segment.get("type", "")
            data = segment.get("data", {})
            
            if seg_type == "text":
                texts.append(data.get("text", ""))
            elif seg_type == "image":
                texts.append("[图片]")
            elif seg_type == "audio":
                texts.append("[语音]")
            elif seg_type == "video":
                texts.append("[视频]")
            elif seg_type == "file":
                texts.append(f"[文件: {data.get('file_name', '')}]")
            elif seg_type == "mention":
                texts.append(f"@{data.get('user_id', '')}")
        
        return " ".join(texts)
    
    def _map_chat_type(self, chat_type: int) -> str:
        """
        映射云湖 chat_type 到 OneBot12 detail_type
        
        :param chat_type: 云湖 chat_type (1=user, 2=group, 3=bot)
        :return: OneBot12 detail_type
        """
        mapping = {
            1: "private",
            2: "group",
            3: "bot"
        }
        return mapping.get(chat_type, "private")
    
    def _generate_event_id(self, event: Dict[str, Any]) -> str:
        """
        生成事件 ID

        :param event: 原始事件
        :return: 事件 ID
        """
        # 尝试从消息中获取 ID
        if "msg" in event.get("data", {}):
            msg = event["data"]["msg"]
            msg_id = msg.get("msg_id", "")
            if msg_id:
                return msg_id

        # 否则生成 UUID
        return str(uuid.uuid4())

    def _convert_file_send_message(self, ws_event: Dict[str, Any]) -> Dict[str, Any]:
        """
        转换超级文件分享事件为 OneBot12 notice 事件

        :param ws_event: file_send_message 事件
        :return: OneBot12 notice 事件
        """
        data = ws_event.get("data", {})
        sender_info = data.get("sender", {})
        info = ws_event.get("info", {})

        # 提取基本信息
        send_user_id = sender_info.get("send_user_id", "")
        user_id = sender_info.get("user_id", "")
        send_type = sender_info.get("send_type", "")
        file_data = sender_info.get("data", "")

        # 构建 OneBot12 事件
        ob12_event = {
            "id": self._generate_event_id(ws_event),
            "time": int(time.time()),
            "type": "notice",
            "detail_type": "yunhu_user_file_send",
            "platform": "yunhu_user",
            "self": {
                "platform": "yunhu_user",
                "user_id": ""  # 由适配器填充
            },
            "user_id": send_user_id,
            "user_nickname": "",
            "yunhu_user_file_send": {
                "send_user_id": send_user_id,
                "user_id": user_id,
                "send_type": send_type,
                "data": file_data
            },
            "yunhu_user_raw": ws_event,
            "yunhu_user_raw_type": "file_send_message"
        }

        return ob12_event

    def _convert_bot_board_message(self, ws_event: Dict[str, Any]) -> Dict[str, Any]:
        """
        转换机器人公告设置事件为 OneBot12 notice 事件

        :param ws_event: bot_board_message 事件
        :return: OneBot12 notice 事件
        """
        data = ws_event.get("data", {})
        board = data.get("board", {})
        info = ws_event.get("info", {})

        # 提取基本信息
        bot_id = board.get("bot_id", "")
        chat_id = board.get("chat_id", "")
        chat_type = board.get("chat_type", 1)
        content = board.get("content", "")
        content_type = board.get("content_type", 1)
        last_update_time = board.get("last_update_time", 0)
        bot_name = board.get("bot_name", "")

        # 转换 detail_type
        detail_type = self._map_chat_type(chat_type)

        # 构建 OneBot12 事件
        ob12_event = {
            "id": self._generate_event_id(ws_event),
            "time": int(time.time()),
            "type": "notice",
            "detail_type": "yunhu_user_bot_board",
            "platform": "yunhu_user",
            "self": {
                "platform": "yunhu_user",
                "user_id": ""  # 由适配器填充
            },
            "bot_id": bot_id,
            "bot_name": bot_name,
            "yunhu_user_bot_board": {
                "bot_id": bot_id,
                "chat_id": chat_id,
                "chat_type": chat_type,
                "content": content,
                "content_type": content_type,
                "last_update_time": last_update_time
            },
            "yunhu_user_raw": ws_event,
            "yunhu_user_raw_type": "bot_board_message"
        }

        # 添加位置信息
        if detail_type == "group":
            ob12_event["group_id"] = chat_id
        elif detail_type == "bot":
            ob12_event["bot_id"] = chat_id

        return ob12_event

    def _convert_stream_message(self, ws_event: Dict[str, Any]) -> Dict[str, Any]:
        """
        转换流式消息事件为 OneBot12 message 事件

        :param ws_event: stream_message 事件
        :return: OneBot12 message 事件
        """
        data = ws_event.get("data", {})
        stream_msg = data.get("msg", {})
        info = ws_event.get("info", {})

        # 提取基本信息
        msg_id = stream_msg.get("msg_id", "")
        recv_id = stream_msg.get("recv_id", "")
        chat_id = stream_msg.get("chat_id", "")
        content = stream_msg.get("content", "")

        # 流式消息通常是私聊或群聊，这里默认为私聊
        detail_type = "private"

        # 构建消息段
        message_segments = [{"type": "text", "data": {"text": content}}]
        alt_message = content

        # 构建 OneBot12 事件
        ob12_event = {
            "id": msg_id or self._generate_event_id(ws_event),
            "time": int(time.time()),
            "type": "message",
            "detail_type": detail_type,
            "platform": "yunhu_user",
            "self": {
                "platform": "yunhu_user",
                "user_id": ""  # 由适配器填充
            },
            "message": message_segments,
            "alt_message": alt_message,
            "user_id": recv_id,
            "user_nickname": "",
            "yunhu_user_raw": ws_event,
            "yunhu_user_raw_type": "stream_message"
        }

        # 如果有 chat_id，可能是群聊
        if chat_id:
            ob12_event["chat_id"] = chat_id

        if msg_id:
            ob12_event["message_id"] = msg_id

        return ob12_event


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: msg.proto
# Protobuf Python Version: 6.33.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    5,
    '',
    'msg.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tmsg.proto\x12\x06yh_msg\"7\n\x06Status\x12\x12\n\nrequest_id\x18\x01 \x01(\x03\x12\x0c\n\x04\x63ode\x18\x02 \x01(\x05\x12\x0b\n\x03msg\x18\x03 \x01(\t\".\n\x03Tag\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0c\n\x04text\x18\x03 \x01(\t\x12\r\n\x05\x63olor\x18\x04 \x01(\t\"\xf3\x07\n\x03Msg\x12\x0e\n\x06msg_id\x18\x01 \x01(\t\x12\"\n\x06sender\x18\x02 \x01(\x0b\x32\x12.yh_msg.Msg.Sender\x12\x11\n\tdirection\x18\x03 \x01(\t\x12\x14\n\x0c\x63ontent_type\x18\x04 \x01(\x05\x12$\n\x07\x63ontent\x18\x05 \x01(\x0b\x32\x13.yh_msg.Msg.Content\x12\x11\n\tsend_time\x18\x06 \x01(\x03\x12\x1c\n\x03\x63md\x18\x07 \x01(\x0b\x32\x0f.yh_msg.Msg.Cmd\x12\x17\n\x0fmsg_delete_time\x18\x08 \x01(\x03\x12\x14\n\x0cquote_msg_id\x18\t \x01(\t\x12\x0f\n\x07msg_seq\x18\n \x01(\x03\x12\x11\n\tedit_time\x18\x0c \x01(\x03\x1a!\n\x03\x43md\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x04 \x01(\x05\x1a\xc6\x04\n\x07\x43ontent\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x0f\n\x07\x62uttons\x18\x02 \x01(\t\x12\x11\n\timage_url\x18\x03 \x01(\t\x12\x11\n\tfile_name\x18\x04 \x01(\t\x12\x10\n\x08\x66ile_url\x18\x05 \x01(\t\x12\x0c\n\x04\x66orm\x18\x07 \x01(\t\x12\x16\n\x0equote_msg_text\x18\x08 \x01(\t\x12\x13\n\x0bsticker_url\x18\t \x01(\t\x12\x0f\n\x07post_id\x18\n \x01(\t\x12\x12\n\npost_title\x18\x0b \x01(\t\x12\x14\n\x0cpost_content\x18\x0c \x01(\t\x12\x19\n\x11post_content_type\x18\r \x01(\t\x12\x15\n\rexpression_id\x18\x0f \x01(\t\x12\x17\n\x0fquote_image_url\x18\x10 \x01(\t\x12\x18\n\x10quote_image_name\x18\x11 \x01(\t\x12\x11\n\tfile_size\x18\x12 \x01(\x03\x12\x11\n\tvideo_url\x18\x13 \x01(\t\x12\x11\n\taudio_url\x18\x15 \x01(\t\x12\x12\n\naudio_time\x18\x16 \x01(\x03\x12\x17\n\x0fquote_video_url\x18\x17 \x01(\t\x12\x18\n\x10quote_video_time\x18\x18 \x01(\x03\x12\x17\n\x0fsticker_item_id\x18\x19 \x01(\x03\x12\x17\n\x0fsticker_pack_id\x18\x1a \x01(\x03\x12\x11\n\tcall_text\x18\x1d \x01(\t\x12\x18\n\x10\x63\x61ll_status_text\x18  \x01(\t\x12\r\n\x05width\x18! \x01(\x03\x12\x0e\n\x06height\x18\" \x01(\x03\x12\x0b\n\x03tip\x18% \x01(\t\x1ay\n\x06Sender\x12\x0f\n\x07\x63hat_id\x18\x01 \x01(\t\x12\x11\n\tchat_type\x18\x02 \x01(\x05\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x12\n\navatar_url\x18\x04 \x01(\t\x12\x0f\n\x07tag_old\x18\x06 \x03(\t\x12\x18\n\x03tag\x18\x07 \x03(\x0b\x32\x0b.yh_msg.Tag\"O\n\x18list_message_by_seq_send\x12\x0f\n\x07msg_seq\x18\x03 \x01(\x03\x12\x11\n\tchat_type\x18\x04 \x01(\x03\x12\x0f\n\x07\x63hat_id\x18\x05 \x01(\t\"^\n\x13list_message_by_seq\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.yh_msg.Status\x12\x18\n\x03msg\x18\x02 \x03(\x0b\x32\x0b.yh_msg.Msg\x12\r\n\x05total\x18\x03 \x01(\x05\"\x87\x01\n\x1clist_message_by_mid_seq_send\x12\x0f\n\x07msg_seq\x18\x03 \x01(\x03\x12\x11\n\tchat_type\x18\x04 \x01(\x03\x12\x0f\n\x07\x63hat_id\x18\x05 \x01(\t\x12\x0f\n\x07unknown\x18\x06 \x01(\x03\x12\x11\n\tmsg_count\x18\x07 \x01(\x03\x12\x0e\n\x06msg_id\x18\x08 \x01(\t\"b\n\x17list_message_by_mid_seq\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.yh_msg.Status\x12\x18\n\x03msg\x18\x02 \x03(\x0b\x32\x0b.yh_msg.Msg\x12\r\n\x05total\x18\x03 \x01(\x05\"Z\n\x11list_message_send\x12\x11\n\tmsg_count\x18\x02 \x01(\x03\x12\x0e\n\x06msg_id\x18\x03 \x01(\t\x12\x11\n\tchat_type\x18\x04 \x01(\x03\x12\x0f\n\x07\x63hat_id\x18\x05 \x01(\t\"H\n\x0clist_message\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.yh_msg.Status\x12\x18\n\x03msg\x18\x02 \x03(\x0b\x32\x0b.yh_msg.Msg\"\xd7\x06\n\x11send_message_send\x12\x0e\n\x06msg_id\x18\x02 \x01(\t\x12\x0f\n\x07\x63hat_id\x18\x03 \x01(\t\x12\x11\n\tchat_type\x18\x04 \x01(\x03\x12\x32\n\x07\x63ontent\x18\x05 \x01(\x0b\x32!.yh_msg.send_message_send.Content\x12\x14\n\x0c\x63ontent_type\x18\x06 \x01(\x03\x12\x12\n\ncommand_id\x18\x07 \x01(\x03\x12\x14\n\x0cquote_msg_id\x18\x08 \x01(\t\x12.\n\x05media\x18\t \x01(\x0b\x32\x1f.yh_msg.send_message_send.Media\x1a\xc1\x03\n\x07\x43ontent\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x0f\n\x07\x62uttons\x18\x02 \x01(\t\x12\x11\n\tfile_name\x18\x04 \x01(\t\x12\x0c\n\x04\x66ile\x18\x05 \x01(\t\x12\x14\n\x0cmentioned_id\x18\x06 \x03(\t\x12\x0c\n\x04\x66orm\x18\x07 \x01(\t\x12\x16\n\x0equote_msg_text\x18\x08 \x01(\t\x12\r\n\x05image\x18\t \x01(\t\x12\x11\n\tmsg_text1\x18\x0b \x01(\t\x12\x11\n\tmsg_text2\x18\x0c \x01(\t\x12\x15\n\rexpression_id\x18\x0f \x01(\t\x12\x17\n\x0fquote_image_url\x18\x10 \x01(\t\x12\x18\n\x10quote_image_name\x18\x11 \x01(\t\x12\x11\n\tfile_size\x18\x12 \x01(\x03\x12\r\n\x05video\x18\x13 \x01(\t\x12\r\n\x05\x61udio\x18\x15 \x01(\t\x12\x12\n\naudio_time\x18\x16 \x01(\x03\x12\x17\n\x0fquote_video_url\x18\x17 \x01(\t\x12\x18\n\x10quote_video_time\x18\x18 \x01(\x03\x12\x17\n\x0fsticker_item_id\x18\x19 \x01(\x03\x12\x17\n\x0fsticker_pack_id\x18\x1a \x01(\x03\x12\x11\n\troom_name\x18\x1d \x01(\t\x1a\xa5\x01\n\x05Media\x12\x10\n\x08\x66ile_key\x18\x01 \x01(\t\x12\x11\n\tfile_hash\x18\x02 \x01(\t\x12\x11\n\tfile_type\x18\x03 \x01(\t\x12\x14\n\x0cimage_height\x18\x05 \x01(\x03\x12\x13\n\x0bimage_width\x18\x06 \x01(\x03\x12\x11\n\tfile_size\x18\x07 \x01(\x03\x12\x11\n\tfile_key2\x18\x08 \x01(\t\x12\x13\n\x0b\x66ile_suffix\x18\t \x01(\t\".\n\x0csend_message\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.yh_msg.Status\"E\n\x0frecall_msg_send\x12\x0e\n\x06msg_id\x18\x02 \x01(\t\x12\x0f\n\x07\x63hat_id\x18\x03 \x01(\t\x12\x11\n\tchat_type\x18\x04 \x01(\x03\",\n\nrecall_msg\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.yh_msg.Status\"K\n\x15recall_msg_batch_send\x12\x0e\n\x06msg_id\x18\x02 \x03(\t\x12\x0f\n\x07\x63hat_id\x18\x03 \x01(\t\x12\x11\n\tchat_type\x18\x04 \x01(\x03\"2\n\x10recall_msg_batch\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.yh_msg.Status\"\xe9\x01\n\x11\x65\x64it_message_send\x12\x0e\n\x06msg_id\x18\x02 \x01(\t\x12\x0f\n\x07\x63hat_id\x18\x03 \x01(\t\x12\x11\n\tchat_type\x18\x04 \x01(\x05\x12\x32\n\x07\x63ontent\x18\x05 \x01(\x0b\x32!.yh_msg.edit_message_send.Content\x12\x14\n\x0c\x63ontent_type\x18\x06 \x01(\x03\x12\x14\n\x0cquote_msg_id\x18\x08 \x01(\t\x1a@\n\x07\x43ontent\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x0f\n\x07\x62uttons\x18\x02 \x01(\t\x12\x16\n\x0equote_msg_text\x18\x08 \x01(\t\".\n\x0c\x65\x64it_message\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.yh_msg.Status\"o\n\x12\x62utton_report_send\x12\x0e\n\x06msg_id\x18\x02 \x01(\t\x12\x11\n\tchat_type\x18\x03 \x01(\x03\x12\x0f\n\x07\x63hat_id\x18\x04 \x01(\t\x12\x0f\n\x07user_id\x18\x05 \x01(\t\x12\x14\n\x0c\x62utton_value\x18\x06 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'msg_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_STATUS']._serialized_start=21
  _globals['_STATUS']._serialized_end=76
  _globals['_TAG']._serialized_start=78
  _globals['_TAG']._serialized_end=124
  _globals['_MSG']._serialized_start=127
  _globals['_MSG']._serialized_end=1138
  _globals['_MSG_CMD']._serialized_start=397
  _globals['_MSG_CMD']._serialized_end=430
  _globals['_MSG_CONTENT']._serialized_start=433
  _globals['_MSG_CONTENT']._serialized_end=1015
  _globals['_MSG_SENDER']._serialized_start=1017
  _globals['_MSG_SENDER']._serialized_end=1138
  _globals['_LIST_MESSAGE_BY_SEQ_SEND']._serialized_start=1140
  _globals['_LIST_MESSAGE_BY_SEQ_SEND']._serialized_end=1219
  _globals['_LIST_MESSAGE_BY_SEQ']._serialized_start=1221
  _globals['_LIST_MESSAGE_BY_SEQ']._serialized_end=1315
  _globals['_LIST_MESSAGE_BY_MID_SEQ_SEND']._serialized_start=1318
  _globals['_LIST_MESSAGE_BY_MID_SEQ_SEND']._serialized_end=1453
  _globals['_LIST_MESSAGE_BY_MID_SEQ']._serialized_start=1455
  _globals['_LIST_MESSAGE_BY_MID_SEQ']._serialized_end=1553
  _globals['_LIST_MESSAGE_SEND']._serialized_start=1555
  _globals['_LIST_MESSAGE_SEND']._serialized_end=1645
  _globals['_LIST_MESSAGE']._serialized_start=1647
  _globals['_LIST_MESSAGE']._serialized_end=1719
  _globals['_SEND_MESSAGE_SEND']._serialized_start=1722
  _globals['_SEND_MESSAGE_SEND']._serialized_end=2577
  _globals['_SEND_MESSAGE_SEND_CONTENT']._serialized_start=1960
  _globals['_SEND_MESSAGE_SEND_CONTENT']._serialized_end=2409
  _globals['_SEND_MESSAGE_SEND_MEDIA']._serialized_start=2412
  _globals['_SEND_MESSAGE_SEND_MEDIA']._serialized_end=2577
  _globals['_SEND_MESSAGE']._serialized_start=2579
  _globals['_SEND_MESSAGE']._serialized_end=2625
  _globals['_RECALL_MSG_SEND']._serialized_start=2627
  _globals['_RECALL_MSG_SEND']._serialized_end=2696
  _globals['_RECALL_MSG']._serialized_start=2698
  _globals['_RECALL_MSG']._serialized_end=2742
  _globals['_RECALL_MSG_BATCH_SEND']._serialized_start=2744
  _globals['_RECALL_MSG_BATCH_SEND']._serialized_end=2819
  _globals['_RECALL_MSG_BATCH']._serialized_start=2821
  _globals['_RECALL_MSG_BATCH']._serialized_end=2871
  _globals['_EDIT_MESSAGE_SEND']._serialized_start=2874
  _globals['_EDIT_MESSAGE_SEND']._serialized_end=3107
  _globals['_EDIT_MESSAGE_SEND_CONTENT']._serialized_start=3043
  _globals['_EDIT_MESSAGE_SEND_CONTENT']._serialized_end=3107
  _globals['_EDIT_MESSAGE']._serialized_start=3109
  _globals['_EDIT_MESSAGE']._serialized_end=3155
  _globals['_BUTTON_REPORT_SEND']._serialized_start=3157
  _globals['_BUTTON_REPORT_SEND']._serialized_end=3268
# @@protoc_insertion_point(module_scope)



import aiohttp
from typing import Dict, Any, Optional


class YunhuHTTPClient:

    BASE_URL = "https://chat-go.jwzhd.com/v1"
    
    def __init__(self, token: Optional[str] = None, timeout: int = 30):
        """
        初始化 HTTP 客户端

        :param token: 用户 token
        :param timeout: 请求超时时间（秒）
        """
        self.token = token
        self.timeout = aiohttp.ClientTimeout(total=timeout)
        self._client: Optional[aiohttp.ClientSession] = None
        # 添加 logger
        from ErisPulse.Core import logger
        self.logger = logger

        # 添加别名方法
        self.list = self.list_msg
        self.list_by_seq = self.list_msg_by_seq
        self.list_by_mid_seq = self.list_msg_by_mid_seq
        self.send = self.send_msg
        self.recall = self.recall_msg
        self.recall_batch = self.recall_msg_batch
        self.edit = self.edit_msg
        self.edit_record = self.list_msg_edit_record
    
    @property
    def client(self) -> aiohttp.ClientSession:
        """获取或创建客户端会话"""
        if self._client is None or self._client.closed:
            self._client = aiohttp.ClientSession(timeout=self.timeout)
        return self._client
    
    def _get_headers(self) -> Dict[str, str]:
        """获取请求头"""
        headers = {"Content-Type": "application/json"}
        if self.token:
            headers["token"] = self.token
        return headers
    
    def set_token(self, token: str):
        self.token = token
    
    async def email_login(self, email: str, password: str, 
                       device_id: str, platform: str = "windows") -> Dict[str, Any]:
        """
        邮箱登录
        
        :param email: 邮箱
        :param password: 密码
        :param device_id: 设备ID
        :param platform: 平台
        :return: 登录响应
        """
        url = f"{self.BASE_URL}/user/email-login"
        payload = {
            "email": email,
            "password": password,
            "deviceId": device_id,
            "platform": platform
        }
        
        async with self.client.post(url, json=payload) as response:
            response.raise_for_status()
            return await response.json()
    
    async def get_user_info(self) -> Dict[str, Any]:
        """
        获取用户信息
        
        :return: 用户信息响应
        """
        from ..proto.user_pb2 import user_info_response
        
        url = f"{self.BASE_URL}/user/info"
        async with self.client.get(url, headers=self._get_headers()) as response:
            response.raise_for_status()
            
            # 解析 Protobuf 响应
            resp = user_info_response()
            resp.ParseFromString(await response.read())
            
            return {
                "status": resp.status,
                "data": resp.data
            }
    
    async def list_msg(self, chat_id: str, chat_type: int,
                      msg_count: int = 1, msg_id: str = "") -> Dict[str, Any]:
        """
        获取消息列表
        
        :param chat_id: 会话ID
        :param chat_type: 会话类型 (1=user, 2=group, 3=bot)
        :param msg_count: 获取消息数量
        :param msg_id: 从指定消息ID开始（不包含此消息）
        :return: 消息列表响应
        """
        from ..proto.msg_pb2 import list_message_send, list_message
        from google.protobuf import json_format

        request = list_message_send()
        request.chat_id = chat_id
        request.chat_type = chat_type
        request.msg_count = msg_count
        request.msg_id = msg_id

        url = f"{self.BASE_URL}/msg/list-message"
        data = request.SerializeToString()

        async with self.client.post(url, headers=self._get_headers(), data=data) as response:
            response.raise_for_status()

            resp = list_message()
            resp.ParseFromString(await response.read())

            return json_format.MessageToDict(resp, preserving_proto_field_name=True)

    async def list_msg_by_seq(self, chat_id: str, chat_type: int,
                             msg_start: int = 0) -> Dict[str, Any]:
        """
        通过序列获取消息
        
        :param chat_id: 会话ID
        :param chat_type: 会话类型 (1=user, 2=group, 3=bot)
        :param msg_start: 开始的消息序列
        :return: 消息列表响应
        """
        from ..proto.msg_pb2 import list_message_by_seq_send, list_message_by_seq
        from google.protobuf import json_format

        request = list_message_by_seq_send()
        request.chat_id = chat_id
        request.chat_type = chat_type
        request.msg_seq = msg_start

        url = f"{self.BASE_URL}/msg/list-message-by-seq"
        data = request.SerializeToString()

        async with self.client.post(url, headers=self._get_headers(), data=data) as response:
            response.raise_for_status()

            resp = list_message_by_seq()
            resp.ParseFromString(await response.read())

            return json_format.MessageToDict(resp, preserving_proto_field_name=True)

    async def list_msg_by_mid_seq(self, chat_id: str, chat_type: int,
                                msg_id: str = "", msg_count: int = 1,
                                msg_seq: int = -1) -> Dict[str, Any]:
        """
        通过消息ID和序列获取消息
        
        :param chat_id: 会话ID
        :param chat_type: 会话类型 (1=user, 2=group, 3=bot)
        :param msg_id: 开始的消息ID（包含此消息）
        :param msg_count: 获取的消息数量
        :param msg_seq: 从指定的msg_seq开始
        :return: 消息列表响应
        """
        from ..proto.msg_pb2 import list_message_by_mid_seq_send, list_message_by_mid_seq
        from google.protobuf import json_format

        request = list_message_by_mid_seq_send()
        request.chat_id = chat_id
        request.chat_type = chat_type
        request.msg_id = msg_id
        request.msg_count = msg_count
        request.msg_seq = msg_seq

        url = f"{self.BASE_URL}/msg/list-message-by-mid-seq"
        data = request.SerializeToString()

        async with self.client.post(url, headers=self._get_headers(), data=data) as response:
            response.raise_for_status()

            resp = list_message_by_mid_seq()
            resp.ParseFromString(await response.read())

            return json_format.MessageToDict(resp, preserving_proto_field_name=True)

    async def list_msg_edit_record(self, msg_id: str,
                                   size: int = 10, page: int = 1) -> Dict[str, Any]:
        """
        获取消息编辑记录
        
        :param msg_id: 消息ID
        :param size: 每页消息数
        :param page: 获取第N页
        :return: 编辑记录响应
        """
        url = f"{self.BASE_URL}/msg/list-message-edit-record"
        payload = {
            "msgId": msg_id,
            "size": size,
            "page": page
        }

        async with self.client.post(url, headers=self._get_headers(), json=payload) as response:
            response.raise_for_status()
            return await response.json()

    async def send_msg(self, chat_id: str, chat_type: int,
                      msg_id: str, msg_type: int = 1,
                      content: Optional[Dict] = None,
                      quote_msg_id: Optional[str] = None,
                      media: Optional[Dict] = None) -> Dict[str, Any]:
        """
        发送消息（别名方法）

        :param chat_id: 会话ID
        :param chat_type: 会话类型 (1=user, 2=group, 3=bot)
        :param msg_id: 消息ID
        :param msg_type: 消息类型
        :param content: 消息内容
        :param quote_msg_id: 引用消息ID
        :param media: 媒体信息
        :return: 发送响应
        """
        return await self.send_message(chat_id, chat_type, msg_id, msg_type,
                                     content, quote_msg_id, media)

    async def send_message(self, chat_id: str, chat_type: int,
                       msg_id: str, msg_type: int = 1,
                       content: Optional[Dict] = None,
                       quote_msg_id: Optional[str] = None,
                       media: Optional[Dict] = None) -> Dict[str, Any]:
        """
        发送消息

        :param chat_id: 会话ID
        :param chat_type: 会话类型 (1=user, 2=group, 3=bot)
        :param msg_id: 消息ID
        :param msg_type: 消息类型
        :param content: 消息内容
        :param quote_msg_id: 引用消息ID
        :param media: 媒体信息
        :return: 发送响应
        """
        from ..proto.msg_pb2 import send_message_send, send_message
        from google.protobuf import json_format

        from ErisPulse.Core import logger


        logger.debug(f"send_message: chat_id={chat_id}, chat_type={chat_type}, msg_id={msg_id}, msg_type={msg_type}, content={content}, quote_msg_id={quote_msg_id}, media={media}")

        # 构建请求
        request = send_message_send()
        request.msg_id = msg_id
        request.chat_id = chat_id
        request.chat_type = chat_type
        request.content_type = msg_type

        if content:
            content_msg = request.content
            for key, value in content.items():
                if hasattr(content_msg, key):
                    # 特殊处理 repeated 字段 mentioned_id
                    if key == "mentioned_id" and isinstance(value, list):
                        # 清空现有值并扩展新值
                        content_msg.ClearField(key)
                        content_msg.mentioned_id.extend(value)
                    else:
                        setattr(content_msg, key, value)

        if quote_msg_id:
            request.quote_msg_id = quote_msg_id

        if media:
            media_msg = request.media
            for key, value in media.items():
                if hasattr(media_msg, key):
                    setattr(media_msg, key, value)

        # 序列化并发送
        url = f"{self.BASE_URL}/msg/send-message"
        data = request.SerializeToString()

        async with self.client.post(url, headers=self._get_headers(), data=data) as response:
            response.raise_for_status()

            # 解析响应
            resp = send_message()
            resp.ParseFromString(await response.read())

            return json_format.MessageToDict(resp, preserving_proto_field_name=True)
    
    async def edit_msg(self, chat_id: str, chat_type: int, msg_id: str,
                      text: str, msg_type: int = 1,
                      quote_msg_id: Optional[str] = None,
                      buttons: Optional[str] = None) -> Dict[str, Any]:
        """
        编辑消息（别名方法）

        :param chat_id: 会话ID
        :param chat_type: 会话类型
        :param msg_id: 消息ID
        :param text: 新文本内容
        :param msg_type: 消息类型
        :param quote_msg_id: 引用消息ID
        :param buttons: 按钮 JSON 字符串
        :return: 编辑响应
        """
        return await self.edit_message(chat_id, chat_type, msg_id, text, msg_type,
                                    quote_msg_id, buttons)

    async def edit_message(self, chat_id: str, chat_type: int, msg_id: str,
                       text: str, msg_type: int = 1,
                       quote_msg_id: Optional[str] = None,
                       buttons: Optional[str] = None) -> Dict[str, Any]:
        """
        编辑消息

        :param chat_id: 会话ID
        :param chat_type: 会话类型
        :param msg_id: 消息ID
        :param text: 新文本内容
        :param msg_type: 消息类型
        :param quote_msg_id: 引用消息ID
        :param buttons: 按钮 JSON 字符串
        :return: 编辑响应
        """
        from ..proto.msg_pb2 import edit_message_send, edit_message
        from google.protobuf import json_format

        request = edit_message_send()
        request.msg_id = msg_id
        request.chat_id = chat_id
        request.chat_type = chat_type
        request.content_type = msg_type

        content = request.content
        content.text = text
        if quote_msg_id:
            request.quote_msg_id = quote_msg_id
        if buttons:
            content.buttons = buttons

        url = f"{self.BASE_URL}/msg/edit-message"
        data = request.SerializeToString()

        async with self.client.post(url, headers=self._get_headers(), data=data) as response:
            response.raise_for_status()

            resp = edit_message()
            resp.ParseFromString(await response.read())

            return json_format.MessageToDict(resp, preserving_proto_field_name=True)
    
    async def recall_msg(self, chat_id: str, chat_type: int, msg_id: str) -> Dict[str, Any]:
        """
        撤回消息（别名方法）

        :param chat_id: 会话ID
        :param chat_type: 会话类型
        :param msg_id: 消息ID
        :return: 撤回响应
        """
        return await self.recall_message(chat_id, chat_type, msg_id)

    async def recall_message(self, chat_id: str, chat_type: int, msg_id: str) -> Dict[str, Any]:
        """
        撤回消息

        :param chat_id: 会话ID
        :param chat_type: 会话类型
        :param msg_id: 消息ID
        :return: 撤回响应
        """
        from ..proto.msg_pb2 import recall_msg_send, recall_msg
        from google.protobuf import json_format

        request = recall_msg_send()
        request.msg_id = msg_id
        request.chat_id = chat_id
        request.chat_type = chat_type

        url = f"{self.BASE_URL}/msg/recall-msg"
        data = request.SerializeToString()

        async with self.client.post(url, headers=self._get_headers(), data=data) as response:
            response.raise_for_status()

            resp = recall_msg()
            resp.ParseFromString(await response.read())

            return json_format.MessageToDict(resp, preserving_proto_field_name=True)

    async def recall_msg_batch(self, chat_id: str, chat_type: int,
                              msg_id_list: list) -> Dict[str, Any]:
        """
        批量撤回消息

        :param chat_id: 会话ID
        :param chat_type: 会话类型
        :param msg_id_list: 消息ID列表
        :return: 批量撤回响应
        """
        from ..proto.msg_pb2 import recall_msg_batch_send, recall_msg_batch
        from google.protobuf import json_format

        request = recall_msg_batch_send()
        request.chat_id = chat_id
        request.chat_type = chat_type
        request.msg_id.extend(msg_id_list)

        url = f"{self.BASE_URL}/msg/recall-msg-batch"
        data = request.SerializeToString()

        async with self.client.post(url, headers=self._get_headers(), data=data) as response:
            response.raise_for_status()

            resp = recall_msg_batch()
            resp.ParseFromString(await response.read())

            return json_format.MessageToDict(resp, preserving_proto_field_name=True)

    async def button_report(self, chat_id: str, chat_type: int,
                          msg_id: str, user_id: str,
                          button_value: str) -> Dict[str, Any]:
        """
        按钮事件报告

        :param chat_id: 会话ID
        :param chat_type: 会话类型
        :param msg_id: 消息ID
        :param user_id: 用户ID
        :param button_value: 按钮的值
        :return: 按钮事件报告响应
        """
        from ..proto.msg_pb2 import button_report_send
        from google.protobuf import json_format

        request = button_report_send()
        request.msg_id = msg_id
        request.chat_type = chat_type
        request.chat_id = chat_id
        request.user_id = user_id
        request.button_value = button_value

        url = f"{self.BASE_URL}/msg/button-report"
        data = request.SerializeToString()

        async with self.client.post(url, headers=self._get_headers(), data=data) as response:
            response.raise_for_status()

            return json_format.MessageToDict(request, preserving_proto_field_name=True)
    
    async def download_file(self, url: str) -> Optional[bytes]:
        """
        从 URL 下载文件

        :param url: 文件 URL
        :return: 文件二进制数据
        """
        try:
            self.logger.debug(f"正在下载文件: {url}")
            # 使用 allow_redirects=True 自动跟随重定向
            async with self.client.get(url, timeout=aiohttp.ClientTimeout(total=60), allow_redirects=True) as response:
                response.raise_for_status()
                file_data = await response.read()
                content_type = response.headers.get('Content-Type', '')
                final_url = str(response.url)
                self.logger.debug(f"文件下载成功，最终URL: {final_url}, Content-Type: {content_type}, 大小: {len(file_data)} 字节")

                # 检查下载的内容是否过小（可能是错误页面）
                if len(file_data) < 1000:
                    self.logger.warning(f"下载的文件大小过小（{len(file_data)} 字节），可能是错误页面或重定向问题")

                return file_data
        except Exception as e:
            self.logger.error(f"文件下载失败: {e}")
            return None

    async def upload_file(self, file_type: str, file_data: bytes,
                      filename: Optional[str] = None) -> Optional[str]:
        """
        上传文件到七牛云存储

        :param file_type: 文件类型 (image/audio/video/file)
        :param file_data: 文件二进制数据
        :param filename: 文件名
        :return: 文件 key
        """
        import hashlib

        # 获取七牛 token
        token_url = {
            "image": f"{self.BASE_URL}/misc/qiniu-token",
            "audio": f"{self.BASE_URL}/misc/qiniu-token-audio",
            "video": f"{self.BASE_URL}/misc/qiniu-token-video",
            "file": f"{self.BASE_URL}/misc/qiniu-token2"
        }.get(file_type, f"{self.BASE_URL}/misc/qiniu-token")

        self.logger.debug(f"获取七牛 token，URL: {token_url}, 文件类型: {file_type}")

        async with self.client.get(token_url, headers=self._get_headers()) as token_response:
            token_response.raise_for_status()
            token_data = await token_response.json()

        upload_token = token_data.get("data", {}).get("token")
        if not upload_token:
            self.logger.error(f"获取七牛上传 token 失败，响应: {token_data}")
            return None

        self.logger.debug(f"获取七牛 token 成功，bucket: {file_type}")
        
        bucket = {
            "image": "chat68",
            "audio": "chat68-audio",
            "video": "chat68-video",
            "file": "chat68-file",
            "group_disk": "chat68-file"
        }.get(file_type, "chat68-file")

        # 获取上传域名
        access_key = upload_token.split(":")[0]
        qiniu_query_url = f"https://api.qiniu.com/v4/query?ak={access_key}&bucket={bucket}"

        try:
            async with self.client.get(qiniu_query_url) as query_response:
                query_response.raise_for_status()
                query_data = await query_response.json()
                up_host = query_data["hosts"][0]["up"]["domains"][0]
            self.logger.debug(f"获取七牛上传域名成功: {up_host}, bucket: {bucket}")
        except Exception as e:
            self.logger.error(f"获取七牛上传域名失败: {e}")
            return None

        # 生成文件 key
        md5 = hashlib.md5(file_data).hexdigest()
        name = filename or md5
        key = name  # 直接使用文件名或MD5

        self.logger.debug(f"准备上传文件，key: {key}, 文件大小: {len(file_data)} 字节")

        # 根据文件类型设置 MIME 类型
        content_type_map = {
            "image": "image/jpeg",
            "audio": "audio/mpeg",
            "video": "video/mp4",
            "file": "application/octet-stream"
        }
        content_type = content_type_map.get(file_type, "application/octet-stream")

        # 优先使用 filetype 库检测 MIME 类型
        try:
            import filetype
            kind = filetype.guess(file_data)
            if kind and kind.mime:
                content_type = kind.mime
                self.logger.debug(f"使用 filetype 检测到 MIME 类型: {content_type}")
        except Exception as e:
            self.logger.debug(f"filetype 检测失败: {e}")

        # 如果 filetype 检测失败，尝试从文件名推断 MIME 类型
        if content_type == "application/octet-stream" and filename:
            import mimetypes
            guessed_type = mimetypes.guess_type(filename)
            if guessed_type and guessed_type[0]:
                content_type = guessed_type[0]
                self.logger.debug(f"从文件名推断 MIME 类型: {content_type}")

        try:
            import io
            form_data = aiohttp.FormData()
            form_data.add_field('token', upload_token)
            form_data.add_field('key', key)
            form_data.add_field('file', file_data, filename=name, content_type=content_type)

            # 上传文件
            upload_url = f"https://{up_host}"
            self.logger.debug(f"上传文件到: {upload_url}")
            async with self.client.post(upload_url, data=form_data) as upload_response:
                response_text = await upload_response.text()
                self.logger.debug(f"七牛上传响应状态: {upload_response.status}, 响应内容: {response_text}")
                upload_response.raise_for_status()
                upload_result = await upload_response.json()

            self.logger.info(f"文件上传成功，key: {key}")
            return {
                "key": key,
                "hash": upload_result.get("hash", ""),
                "bucket": bucket
            }

        except aiohttp.ClientResponseError as e:
            self.logger.error(f"文件上传失败: {e.status}, message='{e.message}', url='{upload_url}'")
            return None
        except Exception as e:
            self.logger.error(f"文件上传失败: {e}")
            import traceback
            self.logger.debug(traceback.format_exc())
            return None
    
    async def close(self):
        if self._client and not self._client.closed:
            await self._client.close()
